#ifndef ADS1115_H
#define ADS1115_H

#include <Arduino.h>
#include <Wire.h>

class  simpleADC{
  public:
    simpleADC(double vdd) {VDD = vdd; maxSteps = VDD / voltStep;}
    uint16_t getAdc(uint8_t);
    double getVolt(uint8_t);

    uint16_t maxSteps;                        // ADS-resolution on VCC, use this to calc a recistance (do not use 32767!!)
    const double voltStep = 4.096 / 32767.0;  // volt per ADS-step - 16 bits INT = neg.->pos. = -32768->0->32767 steps, we use only positive voltages (32768 steps)

  private:
    double VDD;                               // measured VCC, for VDD ~3.3V VDD, gain 4.096V  http://www.ti.com/lit/ds/symlink/ads1115.pdf pag. 19
    /** write buffer**/                                // config register, http://www.ti.com/lit/ds/symlink/ads1115.pdf pag. 19
    uint8_t   writeADS[2] = {0b11000010, 0b10000101},  // bits: MSB (15-8), LSB (7-0)
                           //   a^^b^^                 // ADS write buffer, OR-ing writeADS[0] with portADS[portnr]
                                                       // a) bits 14-12: set port A0-A3 (see portADS[])
                                                       // b) bits 11- 9: set gain amplifier (001 = 4.096V)
    /** up to four ADS1115 modules **/                 // module addresses: http://www.ti.com/lit/ds/symlink/ads1115.pdf pag. 17
               adressADS = 0b1001000,                  // ADDR-GND 0x48, ADDR-VDD 0x49, ADDR-SDA 0x4A, ADDR-SCL 0x4B
    /** up to four analog ports per ADS1115 module **/ // config register: http://www.ti.com/lit/ds/symlink/ads1115.pdf pag. 19
               portADS[4] = {0b01000000, 0b01010000, 0b01100000, 0b01110000};  // A0, A1, A2, A3
                          //    ^^^         ^^^         ^^^         ^^^           bits of ports A0-A3
                                                       // now you can use ports 
                                                       //   0-3 by 1 module
                                                       //   0-7 by 2 modules
                                                       //   0-11 by 3 modules
                                                       //   0-15 by 4 modules
};

#endif
