#include <simpleDB_SPIFFS.h>
  /* =============================================================================
   * UN-comment "#define FORMAT_SPIFFS" to: format SPIFFS-mem. and create a new DB
   * COMMENT this if using a early created DB
   * ============================================================================= */
//  #define FORMAT_SPIFFS  // delete al your files ???

/** define the user db record and fields parameters **/
const char   nameDB[31] = "/demo.db",   // if using SPIFFS then the '/' is necessary
           backupDB[31] = "/demo.cdb";  // backup to SD or SPIFFS


/** add/remove fields and set width and the field names **/
/** simpleDB record format:
 *  key.|T|field1....|field2.........|field3...|....ad/remove fields.....|
 *  | = fields delimiter
 *  T = tombstone flag
 */
const uint8_t keyLen     = 4,         // set the FIXED key len
              field1Len = 10,         // set max field len ...
              field2Len = 20,         // ... add/remove fields
              field3Len = 30,
              field4Len = 40,
              recLen =  keyLen + 1 +  // len plus one fields delimiter position
                                 2 +  // FIXED tombstone flag plus one delimiter position
                     field1Len + 1 +  // add/remove field len ...
                     field2Len + 1 +  //   ... must correspond with the above declared nummbers field<?>Len fields
                     field3Len + 1 +
                     field4Len + 1;
char recordStr[recLen + 1];           // to set the read/write buffer
const uint16_t maxRec = 512;          // max number of records

typedef struct RecFields{  /** define the table value holders **/
    char key[keyLen + 1];             // do NOT change the name "key"
    char field1[field1Len + 1];       // change te name and add/remove fields ...
    char field2[field2Len + 1];       //   ... must correspond with the above declared nummbers field<?>Len fields
    char field3[field3Len + 1];   
    char field4[field4Len + 1];
};  

/** the list of simpleDB funtions
 *  =============================
    ** specific simpleDB funtions **
    uint8_t dtW;   // max data width in a record, exclusive 'key' and 'tombstone' marker
    uint16_t dbR;  // record counter, exlusive tombstone records
    char *dbB;     // record read buffer

    ** init an new or existing database **
    DB_RESULT initDB(const char *nameDB, const char *backupDB, uint8_t keyLen, uint8_t recLen, uint16_t maxRec);
    ** simpleDB functions **
    DB_RESULT nrRec();                                      // set counter dbR, the nummer of records, tombstone records are excluded
    DB_RESULT addRec(const char *key, const char *data);    // add a record with a UNIQUE key 
    DB_RESULT writeRec(const char *key, const char *data);  // save a record, overwrite if key exist else add a record 
    DB_RESULT readRec(const char *key);                     // read the record with the given "key", it is in record buffer dbB 
    DB_RESULT getRec(uint8_t nr);                           // read a record on position "nr", it is in record buffer dbB 
    DB_RESULT findRec(const char *key);                     // find record with the given "key", slow but simple and fast enough for a small db 
    DB_RESULT delRec(const char *key);                      // tombstone a record, remove tombstones with comprDb 
    DB_RESULT cmprDb();                                     // copy db, then remove tombstone records 
    DB_RESULT openDB();                                     // test if open, else open or create DB
              closeDB();                                    // close db
    
    DB_RESULT dbResult;                                     // DB_RESULT error code holder
    
    enum DB_RESULT {  // result code
      DB_OK       =  0,  // OK, no error
      DB_EMPTY    =  1,  // no record(s) in db
      NOT_FOUND   =  2,  // record not found
      REC_EXIST   =  3,  // record already exists
      // ERRORs //
      DIF_NR      =  4,  // difference in number of records (DB and backup DB)
      CORRUPT     =  5,  // db corrupt
      MAX_CHARS   =  6,  // key or data: too many characters
      MAX_REC     =  7,  // max records in db
      SEEK_ERR    =  8,  // file seek error  
      EXIST_ERR   =  9,  // no file
      BACKUP_ERR  = 10,  // backup problem
      FORMAT_ERR  = 11,  // format problem
      TOMBSTONE   = 12,  // deleted record
      REMOVE_ERR  = 13,  // delete file error
      RENAME_ERR  = 14,  // rename file error
      MOUNT_ERR   = 15,  // can not mount  
      OPEN_ERR    = 16,  // file open error  
      DB_ERR      = 17,  // db error
};
**/  

simpleDB db;       // db
RecFields recIn;   // IN buffer structure (write)
RecFields recOut;  // OUT buffer structure (read)
DB_RESULT result;  // DB functions result holder (OK, ERROR, ...)

void setRec(char *_key, char *_field1, char *_field2, char *_field3, char *_field4);
DB_RESULT dbWriteRec();
DB_RESULT dbReadRec();

char codeList[18][11] = {
  "DB_OK",
  "DB_EMPTY",
  "NOT_FOUND",
  "REC_EXIST",
  "DIF_NR",
  "CORRUPT",
  "MAX_CHARS",
  "MAX_REC",
  "SEEK_ERR",
  "EXIST_ERR",
  "BACKUP_ERR",
  "FORMAT_ERR",
  "TOMBSTONE",
  "REMOVE_ERR",
  "RENAME_ERR",
  "MOUNT_ERR",
  "OPEN_ERR",
  "DB_ERR"
};

void setup() {
  Serial.begin(115200);
  Serial.setDebugOutput(true); 
  Serial.println("\n...\n");

#ifdef FORMAT_SPIFFS
  result = DB_OK;
  printf("format SPIFFS ...");
  if (!SPIFFS.format()) result = FORMAT_ERR;
  printf("result, SPIFFS.format():\t%s\n", codeList[result]);
#endif

  /** for this demo delete a existed db **/
//  result = db.initDB(nameDB, backupDB, keyLen, recLen, maxRec);
//  db.closeDB();
//  SPIFFS.remove((char*)nameDB);
  
  /** init and open dtabase **/
  result = db.initDB(nameDB, backupDB, keyLen, recLen, maxRec);
 
  /** make end save a record using RecFields recIn.<fields> **/
  printf(">>> overwrite or add a record using structure 'recIn' and function 'dbWriteRec'...\n\n");
  setRec("keyA", "field1A", "field2A", "field3A", "field4A"); // set record fields ...
  printf("\trecIn.key:\t%s\n\trecIn.field1:\t%s\n\trecIn.field2:\t%s\n\trecIn.field3:\t%s\n\trecIn.field4:\t%s\n", 
           recIn.key, recIn.field1, recIn.field2, recIn.field3, recIn.field4);
  result = dbWriteRec();  // ... and save it, add or, if key exist, overwrite record
  printf("result, dbWriteRec():\t%s\n\n", codeList[result]);

  /** overwrite or add a record with a string **/
  printf(">>> overwrite or add a record using a 'string' and function 'db.writeRec'...\n\n");
  char str[db.dtW + 1];
  sprintf(str, "%s|%s|%s|%s|", "field1B", "field2B", "field3B", "field4B");
  printf("\"%s\"\n", str);
  result = db.writeRec("keyB", str);  // add or, if key exist, overwrite record
  printf("result, writeRec(\"keyB\"): %s\n\n", codeList[result]);

  /**  add a record  **/
  printf(">>> add a record with function 'db.addRec'...\n\n");
  sprintf(str, "%s|%s|%s|%s|", "field1C", "field2C", "field3C", "field4C");
  printf("\"%s\"\n", str);
  result = db.addRec("keyC", str);  // add or, if key exist then error
  printf("result, addRec(\"keyC\"):\t%s\n\n", codeList[result]);

  printf(">>> # of records in DB 'uint16_t  dbR'...\n\n");
  printf("records in DB:\t%d\n\n", db.dbR); 

  /** find a record **/
  printf(">>> find a record with function 'db.findRec' (test if record exists)...\n\n");
  result = db.findRec("keyB");
  printf("result, findRec(\"keyB\"): %s\n\n", codeList[result]);

  /** get a record (incl tomstone) **/
  printf(">>> get a record (incl tomstone) with function 'db.getRec'...\n\n");
  result = db.getRec(1);
  printf("result, getRec(1):\t%s, %s\n", codeList[result], db.dbB);
  result = db.getRec(2);
  printf("result, getRec(2):\t%s, %s\n", codeList[result], db.dbB);
  result = db.getRec(0);
  printf("result, getRec(0):\t%s, %s\n", codeList[result], db.dbB);
  printf(">>> there is not a record 4...\n\n");
  result = db.getRec(4);
  printf("result, getRec(4):\t%s, %s\n\n", codeList[result], db.dbB);

  /** delete a record (set tomstone) **/
  printf(">>> delete a record (set tomstone) with function 'db.delRec'...\n\n");
  result = db.delRec("keyB");
  printf("result, delRec(\"keyB\"):\t%s\n", codeList[result]);
  printf(">>> tombstone is set to \"1\"...\n\n");
  result = db.getRec(1);
  printf("result, getRec 1:\t%s, %s\n", codeList[result], db.dbB);
  printf(">>> no tombstone in list...\n\n");

  printf(">>> using structure 'recOut' and function 'dbReadRec'...\n\n");
  /** first set the "key" in recOut.key **/
  strcpy(recOut.key, "keyC");
  result = dbReadRec();
  printf("\trecOut.key:\t%s\n\trecOut.field1:\t%s\n\trecOut.field2:\t%s\n\trecOut.field3:\t%s\n\trecOut.field4:\t%s\n", 
          recOut.key, recOut.field1, recOut.field2, recOut.field3, recOut.field4);
  printf("result, dbReadRec():\t%s\n\n", codeList[result]);

  printf(">>> copy db and remove tombstone records with function 'cmprDb'...\n\n");
  result = db.cmprDb();
  printf("result, db.cmprDb():\t%s\n\n", codeList[result]);

  printf(">>> # of records in DB 'uint16_t  dbR'...\n\n");
  printf("records in DB:\t%d\n\n", db.dbR); 
  
  db.closeDB();

  /** list files **/
  printf(">>> list of all the files...\n\n");
  Dir dir = SPIFFS.openDir("/");
  while (dir.next()) {
    printf("%12s\t", dir.fileName().c_str());
    File f = dir.openFile("r");
    printf("%d\n", f.size());
  }

  SPIFFS.end();  // Use this method before updating SPIFFS using OTA

}

void loop() {
}

void setRec(char *_key, char *_field1, char *_field2, char *_field3, char *_field4) {
  strcpy(recIn.key, _key);
  strcpy(recIn.field1, _field1);  // change and add/remove RecFields names
  strcpy(recIn.field2, _field2);
  strcpy(recIn.field3, _field3);
  strcpy(recIn.field4, _field4);
}

/** write a record from RecFields names **/
DB_RESULT dbWriteRec() {
  char str[db.dtW + 1];
  /** get a simpleDB 'record' string and save it **/
  // change and add/remove RecFields names
  sprintf(str, "%s|%s|%s|%s|", recIn.field1, recIn.field2, recIn.field3, recIn.field4);
  return db.writeRec(recIn.key, str);  // add or, if key exist, overwrite record 
}

/** read a record and insert it in RecFields recOut.names **/
DB_RESULT dbReadRec() { 
  char *v, *b;
  uint8_t w = db.dtW;
  if (db.readRec(recOut.key) == DB_OK) { // record is in dbB
    while (db.dbB[--w] == ' ') db.dbB[w] = '\0';  // remove trailing white-space (added by addRec())
    v = strtok_r(db.dbB, "|", &b);
    strcpy(recOut.field1, v);   // change and add/remove RecFields names
    v = strtok_r(NULL, "|", &b);
    strcpy(recOut.field2, v);
    v = strtok_r(NULL, "|", &b);
    strcpy(recOut.field3, v);
    v = strtok_r(NULL, "|", &b);
    strcpy(recOut.field4, v);
  }
  return db.dbResult;
}
