/** functions
    time_t getTimestamp();        // get UNIX timestamp
    void setTimestamp(time_t t);  // set time from UNIX timestamp
    void correctTime(int32_t s);  // time correction in seconds
    char * getMsTimestamp();      // timestamp in millis, 14 digits char string
 **/

#include <TimeLib.h>
#include <Wire.h>
#include <RtcMs.h>  // first time use? set time with "setTimestamp(time_t t)"
                    //   in "setup()" after "setSyncProvider(syncProvider)"

const uint8_t sdaPin = D2,           // pin: SDA
              sclPin = D1;           // pin: SCL
const uint16_t SyncInterval = 1800,  // sync the time interval in seconds
               dstOffset = 3600;     // in seconds to set/get the UTC/GMT or local time
const double timeZone = 1;           // to set/get the UTC/GMT or local time (Lord Howe Island rules use a half-hour time zone ;=))
boolean isDST = false;               // summertime/DST?

RtcMs tms = RtcMs();

time_t syncProvider() {
  return tms.getTimestamp();
}

time_t localTime(time_t t) {
  return t + (uint32_t)(timeZone * 3600 + (isDST ? dstOffset : 0)); 
}

void setup() {
  Serial.begin(115200);
//  Serial.setDebugOutput(true); 
  Serial.println("\n...\n");

  Wire.begin(sdaPin, sclPin);
  
  setSyncProvider(syncProvider);  // set the external time provider
  setSyncInterval(SyncInterval);  // set the number of seconds between re-sync

  /** set RTC time options **/ 
  //tms.setTimestamp(1510490400);  // insert a UNIX timestamp http://www.unixtimestamp.com/index.php
  //tms.correctTime(-10);   // int32_t +/- correction in seconds
}
 
void loop() {
  char msTs[14];  // timestamp String with millis
  strcpy(msTs, tms.getMsTimestamp());  // usable as websocket timestamp or ...
  time_t u = now();
  time_t t = localTime(u);
  printf("Loc time   : %02u-%02u-%04u %02u:%02u:%02u \n", day(t), month(t), year(t), hour(t), minute(t), second(t));
  printf("UTC rtc    : %02u-%02u-%04u %02u:%02u:%02u \n", day(u), month(u), year(u), hour(u), minute(u), second(u));
  printf("UTC Millis$: %s \n\n", msTs);
  delay(10000);
}
