/** simpleHomeSwitch    **
 *  www.kendziorra.nl    *
 *  Ewald sr, 2018       *
 *  totally free to use  *
 *************************
 * switches and settings
 *************************/

//========================================================================//
//#####        Minimum configuration: Node "WS" and node "00"        #####//
//========================================================================//
//#####       A new day starts at 04:00:00 and ends at 03:59:59      #####//
//========================================================================//
//##### All the simple... libraries have a HOW TO use example sketch #####//
//========================================================================//
//#####   Add your own compile directives, functions and commands    #####//
//========================================================================//

/** list of compiler switches in this header file
 =====================
 == General ESP8266 ==
 =====================
 - USE_DEBUG          // using Serial print debug data
 - USE_GENERIC        // if using a generic ESP8266 module (ESP8266-01 or Sonoff)
 =====================
 === Used Node type ==
 =====================
         "WS" Node:     
 - SET_AS_NODE_WS     // Node "WS": using as WebSocket Server and SoftwareSerial/Bluetooth Bridge to Node "00"
     or "00" Node:
 - SET_AS_NODE_00     // Node "00": using as Mesh Node and SoftwareSerial/Bluetooth Bridge to Node "WS"
     else:
 - NO switch          // A Mesh Node with a unique NodeID 01-98     
 =====================
 ==== Node Otions ====
 =====================
 - USE_PUBLISH        // using as a doPublish Node (use it only with Node "01" to "98")
 - USE_KAKU           // using KaKu (CoCo) 433MHz radio
 - USE_DALLAS         // using OneWire Dallas
    - USE_TEMP_DS18B20// using temperature and light DS18B20 and BH1750
 - USE_WIRE           // if using I2C (RTC, TEMP, ADS1115, ....)
    - USE_RTC_DC3231  // using simpleTime RTC DC3231
    - USE_LUX_BH1750  // using temperature and light DS18B20 and BH1750
 - USE_ADC            // using A-ports or 4-16 ADS1115 ADC ports
    - USE_ADC_ADS1115 // using 4-16 ADC ports
    - USE_THERMISTOR  // using a Thermistor
 - USE_ASK_NODE       // asking temperature and light
 - USE_STEPS          // using stepper steps, 28YBJ-48 DC 5V 4 Phase 5 Wire Stepper Motor With ULN2003 Driver Board - 4096 steps
 - USE_DEGREES        // using stepper degrees, 28YBJ-48 DC 5V 4 Phase 5 Wire Stepper Motor With ULN2003 Driver Board - 4096 steps
 **/

//===========================<--- MARKER: used to see the #define positions
#define USE_DEBUG            // if using Serial print debug data
//===========================

//===========================
//#define USE_GENERIC          // if using a generic ESP8266 module (ESP8266-01 or Sonoff)
//===========================
#ifdef USE_GENERIC           //   IDE: generic ESP8266 Module, 1M(64K SPIFFS)
                             // not used for Wemos, NodeMCU and ...
                             // if not using a generic ESP8266 Module
                             //   IDE: ESP8266-NOT-generic Module (Wemos/NodeMCU/???), >=1M(<=1M SPIFFS)
  const uint8_t D0 = 16, D1 = 5, D2 = 4, D3 = 0, D4 = 2, D5 = 14, D6 = 12, D7 = 13, D8 = 15;
#endif
//=========================== 

//===========================
//#define SET_AS_NODE_WS     // Node "WS" (WebSocket): using as WebSocket Server and SoftwareSerial/Bluetooth Bridge to Node "00"
//===========================
#ifndef SET_AS_NODE_WS         // NOT a WS (WebSocket) Node but a Mesh Node00-98
  //===========================
  //#define SET_AS_NODE_00     // Node "00": using as Mesh Node and SoftwareSerial/Bluetooth Bridge to Node "WS"
  //===========================     else: use it as a Mesh Node with a unique NodeID 01-98
                      // sHS USE A OLD PAINLESSMESH VERSION
                      // This old BlackAdder PainlessMesh library is stripped and renamed to myMesh.
                      // So no conflict with a other BlackAdder PainlesMesh library.
  #include <myMesh.h>          // https://gitlab.com/BlackEdder/painlessMesh
  myMesh mesh;                 //   a working version is included in the simpleHomeSwitch library
                               //   =============================================================
                               //   painlessMesh version oct 2017 has problems
                               //   this lib. use a "ArduinoJson" library https://github.com/bblanchon/ArduinoJson
  const uint16_t nodePORT = 5555;  // mesh port
#endif
             
//============================
//===========NodeWS===========
//============================
#ifdef SET_AS_NODE_WS          // nodeID: if data IN has this nodeID the data is exclusief for this node
  const char nodeID[] = "WS";  //  if "WS" then the data is from/to the WebSocket/Bluetooth Node
  #include <ESP8266WiFi.h>         // https://github.com/esp8266/Arduino
  #include <WebSocketsServer.h>    // https://github.com/Links2004/arduinoWebSockets
  #include <NtpMs.h>               // A simple time library with a millis timestamp$ http://kendziorra.nl

  /** login data Websocket indentification **/
  const char     mySSID[] = "edit-edit"@edit_andRemove_this,
                   myPW[] = "edit-edit-edit-edit"@edit_andRemove_this,
             socketName[] = "NodeWS",         // Websocket name set this Name also in the simpleHomeSwitch HTML-code
        ntpServerName[] = "time.google.com";  // URL NTP Server, by example: "nl.pool.ntp.org" or "time.google.com" or ....

  /** create a usable router IP **/
  // we use a static IP address and, if using over WWW, Port forwarding as a server in your router
  // set this IP also in the simpleHomeSwitch HTML-code ==========================================
   IPAddress local_IP(  192, 168, 178, 110)@edit_andRemove_this,   // localPort static IP - set this IP as static in your router!
             gateway_IP(192, 168, 178,   1)@edit_andRemove_this,   // gateway IP (your router login IP)
            subnet_IP( 255, 255, 255,   0);   // standard subnet
  const uint16_t myPORT = 8080;               // Websocket port

//============================
//===========Node00===========
//============================
#elif defined(SET_AS_NODE_00)  //  if "WB" then the data is from/to the WebSocket WebBrowser
  const char nodeID[] = "00";  //  if "00" then the data is from/to the Mesh/Bluetooth Node

//============================
//=========Node01-98==========
//============================
#else                          //  if NOT "WS" or "00" then: set a unique value from "01" to "98" or use two characters (lower/uppercase)
  const char nodeID[] = "11"@edit_andRemove_this;  //  if "01" to "98" then the data is from/to a Mesh Node
#endif                         //  NodeID "99" is reserved, use it to communicate with all the nodes
//============================

#if defined(SET_AS_NODE_WS) || defined(SET_AS_NODE_00)
  const uint8_t txPin = D6,  // you can use direct wired Serial communication between the WEB module and this Master Node
                rxPin = D5;  // if direct wired Serial communication: wires cross connect 
                             //   or use a set of HC-05 Bluetooth<-->Serial Master/Slave modules
                             //   HC-05 how to? library simpleHomeSwitch->examples->Bluetooth... 
#endif

//===========================
//#define USE_PUBLISH          // if using as a doPublish Node (use it only with Node "01" to "98")
//===========================
#ifdef USE_PUBLISH           // if used as a doPublish Node then
  /** RECORD data 
    *  recordkey:  publishKey (example: "PuBl")
    *  pin1:       publishPin (example: "D3")
    *  pin2:       publishPin     "      "
    *  repeat:     2
    *  duration:   0
    *  mode:       0 or 8 (0: manual, 8: night, ON from Sunset to Sunrise) 
    *  if mode 8: It can set ON before Sunset with the Publish button or the Web Browser App.
    *  if mode 8: It switch always OFF on Sunrise.
    **/

  char publishKey[] = "PuBl",        // This is the unit key (DB-record key). Use this key to setup a unit action record
       publishPin[] = "D3";          // If the record publishKey exist then you can use the button to send te on/offPublish ((Not)AtHome or ...) 
                                     //   publishPin as char array!!!! in code translated as a GPIO D-pin 
  const uint8_t greenPin = D7,       // we use a GREEN (offPublish list) and a RED (onPublish list) LED as flash LEDs
                redPin = D5;
  const uint16_t onTimePublish = 100,    // LED ON time
                 offTimePublish = 2000,  // LED OFF time
                 msDebouncePublish = 2000;      // msec. button debounce interrupt trigger time
                 
  /**################# UNIT LIST with command actions #################//
  //  const uint8_t nNodes = 3;  // number of unit commands
  //  char onPublish[nNodes][7][11] {  // "going away" command list
  //// format: toID, func, cmd, data1, data2, data3, data4 
  //          {"05", "0", "0", "Alrm", "1",   " ",   " "},  // alarm LED RED flash
  //          {"01", "0", "0", "Heat", "0",   " ",   " "},  // central heating LOW
  //          {"03", "3", "9", "ABCD", "2",   " ",   " ")   // ABCD AUTO-STOP
  //         };
  //      // add fields
  //  char offPublish[nNodes][7][11] {  // "going away" command list
  //// format: toID, func, cmd, data1, data2, data3, data4 
  //          {"05", "0", "0", "Alrm", "0",   " ",   " "},  // alarm LED Green flash
  //          {"01", "0", "0", "Heat", "1",   " ",   " "},  // central heating HIGH
  //          {"03", "3", "9", "ABCD", "0",   " ",   " ")   // ABCD AUTO-ON
  //         };
  //      // add fields 
         TEST: send nonsense to Web Browser **/
      const uint8_t nNodes = 2;  // number of unit commands
      char onPublish[nNodes][7][11] {   // a "going away" command list
  // format: toID, func, cmd, data1, data2, data3, data4 
             {"WB", "0", "0", "going", "away", " ", " "},  // TEST: send nonsense to Web Browser
             {"WB", "0", "0", "bye",   " ",    " ", " "}   // TEST: send nonsense to Web Browser
            };
      char offPublish[nNodes][7][11] {  // a "come home" command list
  // format: toID, func, cmd, data1, data2, data3, data4 
             {"WB", "0", "0", "come", "home", " ", " "},   // TEST: send nonsense to Web Browser
             {"WB", "0", "0", "hello", " ",   " ", " "}    // TEST: send nonsense to Web Browser
            };
#endif

//===========================
//#define USE_KAKU             // if using KaKu
//===========================
#ifdef USE_KAKU
  #include <kaku.h>    
  uint8_t kakuPin = D4;
  KaKu kaku = KaKu(kakuPin);  // set KaKu D-Pin
#endif

//===========================
//#define USE_DALLAS           //   if using OneWire Dallas
//===========================
#ifdef USE_DALLAS
  //===========================
  //#define USE_TEMP_DS18B20     //   if using a DS18B20 temperature
  //===========================
  #ifdef USE_TEMP_DS18B20
    #include <simpleTempDS18B20.h>       // A simple single DS18B20 Temp library 
    const uint8_t tempPin = D3;          //  DS18B20: One-Wire Digital Temperature Sensor + resistor 4.7KOhm
                                         //  ##### using a Sonoff: use PIN-D2 #####
    double tempCorrection = 0.0;         // temperature correction value (change this if ...)
    simpleTemp Temp1(tempPin,            // oneWirePin
                     tempCorrection);    // temperature correction value (change this if ...)
  #endif
#endif

const boolean fahrenheit = false;  // false: use celsius (change this if using F)

//===========================
//#define USE_WIRE            //  if using I2C (RTC, TEMP, ADS1115, ....)
//===========================
#ifdef USE_WIRE
  #include <Wire.h>
  /** I2C, RTC and BH1750 (Lux) **/
  const uint8_t sdaPin = D2,
                sclPin = D1; 

  #ifndef SET_AS_NODE_WS
    //===========================
    //#define USE_RTC_DC3231       // if using simpleTime
    //===========================
    #ifdef USE_RTC_DC3231
      #include <RtcMs.h>         // A simple DS3231 RTC library
      RtcMs tms;
    #endif
  #endif
  
  //===========================
  //#define USE_LUX_BH1750       // if using BH1750 light
  //===========================
  #ifdef USE_LUX_BH1750
    #include <simpleLuxBH1750.h>         // A simple BH1750 Lux library 
    const uint8_t luxAddr = 0x23;        //  0x23 - BH1750 i2c address
    simpleLux Lux = simpleLux(luxAddr);  // 0x23 - BH1750 i2c address
  #endif
  //===========================
  //#define USE_ADC_ADS1115      // if using 4-16 ADC ports
  //===========================
#endif

//===========================
//#define USE_ADC              // if using A-ports or 4-16 ADS1115 ADC ports
//===========================
#ifdef USE_ADC
  #ifdef USE_ADC_ADS1115
    #include <simpleAdcADS1115.h>   // A simple 0-15 ports ADC
    double vccADC = 3.320;  // measured VCC (you have to measure it by yourself)
                  //=====//
    // with a ADS1115 and VCC ~3.3V: ADC resolution = ~26400
    // if using a ESP8266 or Arduino port: ADC resolution = 1023
    double resADC = vccADC / (4.096 / 32767.0);  // VCC / volt per ADS-step - 16 bits;
    simpleADC adc = simpleADC(vccADC);
  #else  
    double resADC = 1023.0;
  #endif
  double  adcVolt = 0.0;  // calculated voltage
  uint8_t adcPin = A0;    // set the A-pin (use the 'A'), if using ADS115 port A0-A15
  uint16_t adcSteps;
  //===========================
  //#define USE_THERMISTOR       // if using a Thermistor
  //===========================
  #ifdef USE_THERMISTOR
    #include <simpleThermistor.h>  // A simple Thermistor library
    uint32_t adcOhm;
    //##### HOW TO: take a look at my simpleThermistor library example #####//
    // change it with the your own measured values
    // init Thermistor and calc Steinhart-Hart model equation to get the A, B and C coefficients
    // (EXAMPLE VALUES Thermistor resistans by temperature range 0-50*C)
    // use a mulimeter and thermometer to get the right values
    // ======================================================= //
    Thermistor therm(25000.0,  5.0,  // Low resistans on temperature
                     10000.0, 25.0,  // Medium resistans on temperature
                      4000.0, 45.0,  // High resistans on temperature
                     10000.0,        // series resistor on room temperature (use a mulimeter)
                      resADC);       // ADC resolution: Arduino/ESP8266 1023, ADS1115 ~26400
  #endif  
#endif

#if !(defined(SET_AS_NODE_WS) || defined(USE_RTC_DC3231))
  #include <TimeMs.h>                  // A simple Time library, use syncTime from a other Node
  TimeMs tms;
                                //             ADD/REMOVE nodeID with/without RTC-time 
  const char timeAskNodes[7][3] = {"WS", "WS", "00", "01", "02", "03", "99"};  // ask Time from list, last always "99" (99: to all). (start with 2x "WS"!!)
                                                   //   afther "WS", if exist, a Node ID with RTC     ================
  uint8_t timeAskNode = 0;                         //   "WS" = NTP time, if "00" no RTC it's NTP time from "WS"  #endif
#endif

//===========================   DEMO CODE ask for ...
//#define USE_ASK_NODE                   // if asking temperature and light
//===========================            // define your own ask-commands and write your own code
#ifdef USE_ASK_NODE                      // ====================================================
                                         // ##### INSERT: all you need #####//       
  const char askTmpLuxNode[] = "00";     // ##### the Node ID to ask Temp and Lux #####
                                         // ADDED: void ascTmpLux(const char *fromNode);
  void ascTmpLux(const char *fromNode);  // this demo code send a reply with Temp and Lux from Node00 
#endif 

//===========================
//#define USE_STEPS            // if using stepper steps
//===========================
#ifdef USE_STEPS
  #include <simpleStepperSteps.h>  // A simple stepper library
  StepperSteps stepper( D5,   // stepPin1, stepper pins on WeMos/NodeMCU/...  (ESP8266 GPIO 14/12/13/15)
                        D6,   // stepPin2
                        D7,   // stepPin3
                        D8,   // stepPin4
                        D0,   // homePin, microswitch HOME position
                        D3,   // (int16_t) nn>=0: endPin microswitch END or maximun number of rotations
                              //           nn<0: end position in steps (no END microswitch)
                     false,   // stepper mode, false: 512 steps is 360*, true: 4096 steps is 360* 
                         1,   // (uint8_t) stepper speed, delay in ms,  default 1, min: 1, max: 50
                         5);  // (uint16_t) The maximun number of rotations to find the home/end position (1-65535)
#endif 

//===========================
//#define USE_DEGREES          // if using stepper degrees
//===========================
#ifdef USE_DEGREES
  #include <simpleDegreesSteps.h>  // A simple degrees stepper library
  Degrees degr( D5,   // stepPin1, stepper pins on WeMos/NodeMCU/...  (ESP8266 GPIO 14/12/13/15)
                D6,   // stepPin2
                D7,   // stepPin3
                D8,   // stepPin4
                D0,   // homePin, microswitch home position
               264);  // endPos, 270* - 6* -  stop -3* before max potentiometer angle
#endif  

//============================== 
#include <TimeLib.h>            // https://github.com/PaulStoffregen/Time
//==============================  included - http://kendziorra.nl

/** simpleHomeSwitch general used libraries **/
#include <simpleDB.h> 
sHS_DB shsDB;

/** other simple libraries **/
#include <simpleSummerTimeDST.h>  // A simple Summertime/DST library   IS ALWAYS USED
                                  //   IF NOT EU then set DST parameters (below)
                                  //   =========================================
/** set DST/Summer Time rules **/
// =====================================================================
boolean isItDST = true;  // ##### set this on first compile time #####
                         // it's only used on first time RTC set Time
// =====================================================================
double timeZone = 1;       // to set/get the UTC/GMT or local time (Lord Howe Island rules use a half-hour time zone ;=))
int16_t dstOffset = 3600;  // in seconds to set/get the UTC/GMT or local time
DST dst( 
    /** DST/Summer Time rules. Change it for a other area **
     ** ================================================= **/
    /** EU summer time rules:
     *    start: last Sunday in March   at 01:00 add      1h   to local time
     *    end:   last Sunday in October at 01:00 subtract 1h   from local time
     **/
  /*startMonth*/    3,
  /*endMonth*/     10,  // 1 - 12 (jan - dec)
  /*startMonthDay*/ 0,
  /*endMonthDay*/   0,  // 1 - 31, not used = 0 (if using ...WeekDay)
  /*startWeekDay*/  1, 
  /*endWeekDay*/    1,  // 1 -  7 (Sun - Sat), not used = 0 (if using ...MonthDay)
  /*startHour*/     1, 
  /*endHour*/       1,  // 0 - 23 (UTC)
  /*startMinute*/   0, 
  /*endMinute*/     0,  // 0 - 59 (Lord Howe Island rules use a half-hour time zone ;=))
  /*startWeek*/    -1, 
  /*endWeek*/      -1,  // 1 -  4, last week of month: -1 (4 or 5)
  /*dstOffset*/  dstOffset,   // dst/summertime offset in seconds
  /*hemisphere*/ true,  // Northern Hemisphere: true,  Southern Hemisphere: false
  /*timeZone*/   timeZone);    // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))
#include <simpleSunRiseSet.h>  // A simple SunRise/Set library SET YOUR LOCATION (below)
sunRiseSet riseSet(dstOffset,  // dst/summertime offset in seconds
                   52,         // The latitude is preceded by a minus sign ( – ) if it is south of the equator (a positive number implies north)
                   5,          // the longitude is preceded by a minus sign if it is west of the prime meridian (a positive number implies east)
                               //    52, 5: the Centre of the Netherlands
                   timeZone,   // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))
                   90.883);    // zenith:
                               //    offical      = 90.883 degrees (90° 50')
                               //    civil        = 96 degrees
                               //    nautical     = 102 degrees
                               //    astronomical = 108 degrees

