/** simpleHomeSwitch    **
 *  www.kendziorra.nl    *
 *  Ewald sr, 2018       *
 *  totally free to use  *
 *************************
 * function actions code
 *************************/

#include "miscellany.h"

/** function 0: switch units **/
void function0() {
 boolean saveRec = false,
          returning = false;
  shsDB.readRec(inBuf.data[0]);  // load record values in shsDB.record[]
  if (shsDB.db.dbResult == REC_EXIST) {
    switch (atoi(inBuf.command)) {
      case 0: {  // unit switch
        boolean publish = false;
        #ifdef USE_PUBLISH
          if(strcmp(shsDB.record[pin1], publishPin) == 0) {        // Publish button pin?
            publish = true;
            strcpy(shsDB.record[status], inBuf.data[1]);     // arg1: 0(OFF/UP),1(ON/DOWN),2(dim-ON)
            buttonState = atoi(inBuf.data[1]) != 0 ? 1 : 0;  // reverse state
            sendPublish();
            setPublishLED();  // set LEDs
          }
          saveRec = true;
        #endif
        if (!publish) {
          /** is it a StepperSteps unit? **/
          if (strcmp(shsDB.record[pin], "stps") == 0) {
            #ifdef USE_STEPS
              /** it's a StepperSteps unit **/
              uint8_t d = atoi(inBuf.data[1]),
                      r = atoi(shsDB.record[pin2]);
              if (d == 0) stepper.gotoHome();   // find home position (using the maximun number of rotations)
              else if (d == 1) {
                if (r == 0) stepper.gotoEnd();  // find end position (using the maximun number of rotations)
                else stepper.stepTo(r);  // go to step nn
              }
              else if (d == 2) {
                r = atoi(inBuf.data[2]);
                if (d == 2) stepper.stepTo(r);  // go to step nn
              }
              strcpy(shsDB.record[status], inBuf.data[1]);
            #endif
          }
          #ifdef USE_DEGREES
            /** is it a DegreesSteps unit? **/
            else if (strcmp(shsDB.record[pin], "dgrs") == 0) {
              /** it's a DegreesSteps unit **/
              uint8_t d = atoi(inBuf.data[1]);
              if (d == 0) degr.findHome();
              else if (d == 1) degr.toDegree(atoi(shsDB.record[pin2]));  // go to degrees nn
              else if (d == 2) degr.toDegree(atoi(inBuf.data[2]));  // go to degrees nn
              strcpy(shsDB.record[status], inBuf.data[1]);
            }
          #endif
          /** kaku unit? **/
          else if (strlen(shsDB.record[period]) == 1) {  // a KaKu period has a length of 3
            /** it's not a KaKu unit **/
            setPin((atoi(inBuf.data[1]) == 1 ? true : false));
          }
          #ifdef USE_KAKU
            else {
              /** 0/1/2(OFF/ON/DIM), dimvalue(optional) **/
              //  func0Cmd0(inBuf.data[0], inBuf.data[1], inBuf.data[2]);
              strcpy(shsDB.record[status], inBuf.data[1]);     // arg1: 0(OFF/UP),1(ON/DOWN),2(dim-ON)
              strcpy(shsDB.record[dimLevel],  inBuf.data[2]);  // arg2: dim level
              if (strlen(shsDB.record[code]) > 1) kaku.kakuOld(shsDB.record[reversed], shsDB.record[status], shsDB.record[period], shsDB.record[code]);   // KaKu New has no code
              else kaku.kakuNew(shsDB.record[reversed], shsDB.record[status], shsDB.record[period], shsDB.record[address], shsDB.record[dimLevelPresent], shsDB.record[dimLevel], shsDB.record[unit], shsDB.record[groupBit]);
            }
          #endif
          saveRec = true;
          break;
        }
      }
      case 1: {  // repeat last action
        #ifdef USE_PUBLISH
          if(strcmp(shsDB.record[pin1], publishPin) == 0) {  // Publish button pin?
            buttonState = atoi(shsDB.record[status]) != 0 ? 1 : 0;
            sendPublish();
            setPublishLED();  // set LEDs
          }
          else setPin((atoi(shsDB.record[status]) == 1 ? true : false));
        #else
          /** is it a StepperSteps unit? **/
          if (strcmp(shsDB.record[pin], "stps") == 0) {
            #ifdef USE_STEPS

              /** it's a StepperSteps unit **/
              stepper.stepTo(atoi(shsDB.record[status]));  // go to step nn
            #endif
          }
          #ifdef USE_DEGREES
             /** is it a DegreesStep unit? **/
            else if (strcmp(shsDB.record[pin], "dgrs") == 0) {
              /** it's a DegreesStep unit **/
              degr.toDegree(atoi(shsDB.record[status]));  // go to degrees nn
            }
          #endif
          /** kaku unit? **/
          else if (strlen(shsDB.record[period]) == 1) {  // a KaKu period has a length of 3
            /** it's not a KaKu unit **/
            setPin((atoi(shsDB.record[status]) == 1 ? true : false));
          }
          #ifdef USE_KAKU
            else {
              if (strlen(shsDB.record[code]) > 1) kaku.kakuOld(shsDB.record[reversed], shsDB.record[status], shsDB.record[period], shsDB.record[code]);   // KaKu New has no code
              else kaku.kakuNew(shsDB.record[reversed], shsDB.record[status], shsDB.record[period], shsDB.record[address], shsDB.record[dimLevelPresent], shsDB.record[dimLevel], shsDB.record[unit], shsDB.record[groupBit]);
            }
          #endif
        #endif
        break;
      }
      case 2: {  // get status
        setInBufToOutBuf(inBuf.data[0], shsDB.record[status]);
        returning = true;
        break;
      }
    }
    if (returning) sendMessage(setOutStr());
    if (saveRec) shsDB.writeRec(inBuf.data[0]); // save shsDB.record[] values in record
  }
}

/** function 1: new unit record **/
void function1() {
  shsDB.readRec(inBuf.data[0]);  // load record values in shsDB.record[]
  if (shsDB.db.dbResult == NOT_FOUND) shsDB.newRecord();  // get the default values in shsDB.record[]
  switch (atoi(inBuf.command)) {
    #ifdef USE_KAKU
      case  0: {  // setup a KaKu OLD unit
        strcpy(shsDB.record[code],   inBuf.data[1]);
        strcpy(shsDB.record[period], inBuf.data[2]);
        break;
      }
      case  1: {  // setup a KaKu NEW unit, step 1/2
        strcpy(shsDB.record[code], " ");  // OLD/NEW KaKu test char (old has more than one char)
        strcpy(shsDB.record[period],   inBuf.data[1]);
        strcpy(shsDB.record[address],  inBuf.data[2]);
        strcpy(shsDB.record[groupBit], inBuf.data[3]);
        break;
      }
      case  2: {  // setup a KaKu NEW unit, step 2/2
        strcpy(shsDB.record[unit],            inBuf.data[1]);
        strcpy(shsDB.record[dimLevelPresent], inBuf.data[2]);
        strcpy(shsDB.record[dimLevel],        inBuf.data[3]);
        break;
      }
    #endif
    case  3: {  // Pin(s)
      strcpy(shsDB.record[pin1], inBuf.data[1]);
      strcpy(shsDB.record[pin2], inBuf.data[2]);
      break;
    }
    #ifdef USE_STEPS
      case  5: {  // Stepper steps
        strcpy(shsDB.record[pin1], inBuf.data[1]);  // we use pin1 as the stepper 511/4095 steps mode
        strcpy(shsDB.record[pin2], inBuf.data[2]);  // we use pin2 as the stepper endSwitsh or # steps
        strcpy(shsDB.record[pin], "stps");          // we use pin as a stepper steps unit indentification
        break;
      }
    #endif
    #ifdef USE_DEGREES
      case  6: {  // Degrees steps
        strcpy(shsDB.record[pin1], inBuf.data[1]);  // we use pin1 as the stepper degrees
        strcpy(shsDB.record[pin], "dgrs");          // we use pin as a stepper degrees unit indentification
        break;
      }
    #endif
  }
  shsDB.writeRec(inBuf.data[0]);  // save shsDB.record[] values in record
}

/** function 2:  set unit modes **/
void function2() {
  boolean saveRec = true;
  shsDB.readRec(inBuf.data[0]);  // load record values in shsDB.record[]
  if (shsDB.db.dbResult == REC_EXIST) {
    switch (atoi(inBuf.command)) {
      case  0: {  // unitMode
        strcpy(shsDB.record[mode], inBuf.data[1]);
        break;
      }
      case  1: {  // unitTime
        char *b;
        if (strlen(inBuf.data[1]) != 5 || inBuf.data[1][2] != ':' ) {
          strcpy(shsDB.record[hh1], "0");
          strcpy(shsDB.record[mm1], "0");
        }
        else {
          strcpy(shsDB.record[hh1], strtok_r(inBuf.data[1], ":", &b));
          strcpy(shsDB.record[mm1], strtok_r(NULL, ":", &b));
        }
        if (strlen(inBuf.data[2]) != 5 || inBuf.data[2][2] != ':' ) {
          strcpy(shsDB.record[hh2], "0");
          strcpy(shsDB.record[mm2], "0");
        }
        else {
          strcpy(shsDB.record[hh2], strtok_r(inBuf.data[2], ":", &b));
          strcpy(shsDB.record[mm2], strtok_r(NULL, ":", &b));
        }
        break;
      }
      case  2: {  // unitTimer
        strcpy(shsDB.record[counter1], inBuf.data[1]);
        break;
      }
      case  3: {  // unitLux
        strcpy(shsDB.record[lux1], inBuf.data[1]);
        strcpy(shsDB.record[lux2], inBuf.data[2]);
        break;
      }
      case  4: {  // unitTemp
        strcpy(shsDB.record[tmp1], inBuf.data[1]);
        strcpy(shsDB.record[tmp2], inBuf.data[2]);
        break;
      }
      case  5: {  // unitOffset
        strcpy(shsDB.record[riseOffset], inBuf.data[1]);
        strcpy(shsDB.record[setOffset],  inBuf.data[2]);
        break;
      }
      case  6: {  // unitYoyo
        strcpy(shsDB.record[yoyo],inBuf.data[1] );
        break;
      }
      case 7: {  // unitRep
        strcpy(shsDB.record[repetitions], inBuf.data[1]);
        strcpy(shsDB.record[duration],    inBuf.data[2] );
        break;
      }
      case 8: {  // unitBad
        strcpy(shsDB.record[badLux], inBuf.data[1]);
        break;
      }
      case 9: {  // unitStop
        strcpy(shsDB.record[pause], inBuf.data[1]);
        break;
      }
      case 10: {  // unitRvrs
        strcpy(shsDB.record[reversed], inBuf.data[1]);
        break;
      }
      case 11: {  // unitDel
        shsDB.db.delRec(inBuf.data[0]);
        saveRec = false;
        break;
      }
    }
    if (saveRec) shsDB.writeRec(inBuf.data[0]); // save shsDB.record[] values in record
  }
}

/** function 3: various **/
void function3() {

  /** is it a command/request (0) or response (1)? **/
  if (strcmp(inBuf.mode, "0") == 0) {
    switch (atoi(inBuf.command)) {
      case 0: { // send the time?
        char t[11];
        sprintf(t, "%d", now());
        setInBufToOutBuf(t);
        sendMessage(setOutStr());
        break;
      }
      case 1: {  // set time 
        #ifndef SET_AS_NODE_WS
          tms.setTimestamp(atoi(inBuf.data[0]));
          setTime(tms.getTimestamp());
          #ifdef USE_DEBUG
            printf("UTC/GMT Received time: %02u-%02u-%04u %02u:%02u:%02u\n\n", day(), month(), year(), hour(), minute(), second());
          #endif
        #endif
        break;
      }
      case 2: {  // time correction in +/-seconds
        #ifndef SET_AS_NODE_WS
          tms.correctTime(atoi(inBuf.data[0]));
          setTime(tms.getTimestamp());
          #ifdef USE_DEBUG
            printf("UTC/GMT Received time: %02u-%02u-%04u %02u:%02u:%02u\n\n", day(), month(), year(), hour(), minute(), second());
          #endif
        #endif
        break;
      }
      #if defined(USE_TEMP_DS18B20) || defined(USE_LUX_BH1750)
        case 3: { // send the Temp and Lux to...
          char t[] = "      ", l[] = "      ";
          #ifdef USE_TEMP_DS18B20
            double c = Temp1.getTemp();
            c = (!fahrenheit ? c : c * 1.8 + 32.0);  // !fahrenheit ? celsius : fahrenheit
            sprintf(t, "%s", String(c).c_str());
          #endif
          #ifdef USE_LUX_BH1750
            sprintf(l, "%d", Lux.getLux());
          #endif
          setInBufToOutBuf(t, l);
          sendMessage(setOutStr());
          break;
        }
      #endif
      #ifdef USE_ADC
        case 4: {  // send adc or thermistor values
          uint8_t p = pinToA(inBuf.data[0]);
          uint16_t i1 = 0;
          double d1 = 0.0;
          char i2[6],
               d2[11];
          boolean f = true;
          #ifdef USE_ADC_ADS1115
            i1 = adc.getAdc(p);  // the using ADS1115 port
          #else
            i1 = analogRead(pinA[p]);  // the using A-pin
          #endif
          switch (atoi(inBuf.data[1])) {
              case 0: {
                d1 = (double)i1 * (vccADC / resADC);
              break;
            }
            case 1: {
              i1 = resADC;
              d1 = (vccADC / resADC);
              break;
            }
            case 2: {
            #ifdef US_THERMISTOR
              d1 =  (!fahrenheit ? therm.getTempC(i1) : therm.getTempF(i1));
              i1 = therm.ntcR;
            #else
              f = false;
            #endif
              break;
            }
          }
          if (f) {
            sprintf(d2, "%s", String(d1, 6).c_str());
            sprintf(i2, "%d", i1);
          }
          else {
            sprintf(d2, "%s", "-");
            sprintf(i2, "%s", "-");
          }
          setInBufToOutBuf(inBuf.data[0], inBuf.data[1], d2, i2);
          sendMessage(setOutStr());
          break;
        }
      #endif
      case 5: {  // send digital pin value
        uint8_t p = pinToD(inBuf.data[0]),
                s = atoi(inBuf.data[1]),
                m = atoi(inBuf.data[2]);
        switch (s) {
          case 0: {
            char v[2];
            sprintf(v, "%s", (digitalRead(p) ? "HIGH" : "LOW"));
            setInBufToOutBuf(inBuf.data[0], inBuf.data[1], inBuf.data[2], v);
            sendMessage(setOutStr());
            break;
          }
          case 1: {
            digitalWrite(p, m);
            break;
          }
          case 2: {
            pinMode(p, m);
            break;
          }
        }
        break;
      }
      case 6: {  // send sunrise/set times
        char r[9],
             s[9],
             d[7];
        sprintf(r, "%02d:%02d", riseSet.sunRiseHour, riseSet.sunRiseMinute);
        sprintf(s, "%02d:%02d", riseSet.sunSetHour, riseSet.sunSetMinute);
        sprintf(d, "%d", dst.isItDst(utcTime, false));
        setInBufToOutBuf(r, s, d);
        sendMessage(setOutStr());
        break;
      }
      case 99: {  // DB backup and compress
        shsDB.db.cmprDb();
        break;
      }
    }
  }
  else if (strcmp(inBuf.mode, "1") == 0){  // it is a response
    switch (atoi(inBuf.command)) {
      case 0: { // sync the time?
        #if !(defined(SET_AS_NODE_WS) || defined(USE_RTC_DC3231))
          if (timeIsSet && timeIsSync) break;
          #ifdef USE_DEBUG
            printf("timeNode: %s, \n", inBuf.from);
          #endif
          setTime(atoi(inBuf.data[0]));
          timeIsSet = true;
          timeIsSync = true;
          timeAskNode = 0;
          syncCounter = 0;
          utcTime = now();
          locTime = dst.localTime(now(), true);  // get DST/Summertime times and isItDST
          saveMinute = minute();

          /** calc several time data **/
          riseSet.calcSun(utcTime, dst.isItDst(utcTime, false));  // get sunrise/set and DST times
          saveDay = weekday(locTime);
          #ifdef USE_DEBUG
            printf("UTC/GMT time: %02u-%02u-%04u %02u:%02u:%02u \n", day(utcTime), month(utcTime), year(utcTime), hour(utcTime), minute(utcTime), second(utcTime));
            printf("  Local time: %02u-%02u-%04u %02u:%02u:%02u \n\n", day(locTime), month(locTime), year(locTime), hour(locTime), minute(locTime), second(locTime));
          #endif
        #endif
        break;
      }
      case 1: {break;}
      case 2: {  // set Temp and Lux
        temperature = atof(inBuf.data[0]);
        luminance = atoi(inBuf.data[1]);
        #ifdef USE_DEBUG
          printf("Received Temp: %02s, Lux: %02s \n", inBuf.data[0], inBuf.data[1]);
        #endif
        break;
      }
    }
  }
}

