/** simpleHomeSwitch    **
 *  www.kendziorra.nl    *
 *  Ewald sr, 2018       *
 *  totally free to use  *
 *************************
 * miscellany code
 *************************/
 
#include "miscellany.h"

void setupNode() {
  Serial.begin(115200);
//  Serial.setDebugOutput(true); 
  Serial.println("\n...\n");
  #ifndef SET_AS_NODE_WS
    /** MESH start **/
    //  mesh.setDebugMsgTypes(ERROR | DEBUG);
    mesh.init(nodeSSID, nodePW, nodePORT);
    mesh.onReceive(&receivedCallback);
  #endif
  #ifdef USE_WIRE
    Wire.begin(sdaPin, sclPin);
  #endif
  #ifdef USE_KAKU
    pinMode(kakuPin, OUTPUT);  
  #endif
  /** init database **/
  shsDB.dbInit();
  if (shsDB.db.dbResult != DB_OK) {
    #ifdef USE_DEBUG 
      printf("DB open error...");
    #endif
    while(1) delay(10);
  }
  setSyncProvider(syncProvider);  // set the external time provider
  setSyncInterval(syncInterval);  // set the number of seconds between re-sync
  #if defined(SET_AS_NODE_WS) || defined(USE_RTC_DC3231)
    utcTime = now();
    locTime = dst.localTime(utcTime, true);  // get DST/Summertime times and isItDST
    riseSet.calcSun(utcTime, dst.isItDst(utcTime, false));  // get sunrise/set and DST times
    saveMinute = minute(locTime);
    saveDay = weekday(locTime);
    timeIsSet = true;
    #ifdef USE_DEBUG 
      printf("UTC/GMT time: %02u-%02u-%04u %02u:%02u:%02u \n", day(utcTime), month(utcTime), year(utcTime), hour(utcTime), minute(utcTime), second(utcTime));
      printf("  Local time: %02u-%02u-%04u %02u:%02u:%02u \n\n", day(locTime), month(locTime), year(locTime), hour(locTime), minute(locTime), second(locTime));
    #endif
  #endif
}

void loopNode() {
  #ifndef SET_AS_NODE_WS
     mesh.update();  // can sync the system time
     if (nodeMessage) {
       nodeMessage = false;
       nodeReceived();  // a new node message?
     }
  #endif  
  if(timeIsSet) {
    if(doStart) unitStart();  // only once after time is sync
    /** ask other Nodes for ??? 10 sec before do actions **/
    #ifdef USE_ASK_NODE  
      /** ask  Temp and Lux from Node ?? **/
      if(askTimer.tmrTime()) {
        ascTmpLux(askTmpLuxNode);
        askTimer.tmrReset(50000, false);  // set 50 sec. interval, stop
      }
    #endif
    /** 1 minute timer, action time? **/
    if (saveMinute != minute()) {
      #ifdef USE_PUBLISH
        /** (re)set button pin **/
        pinMode(pinToD(publishPin), 2);
      #endif
      askTimer.tmrStart(true); 
      #if !(defined(SET_AS_NODE_WS) || defined(USE_RTC_DC3231))
       /** ask for sync Time? **/
       if (++syncCounter == syncMinutes) {
         syncCounter = syncMinutes - 1;  // if time received: it is set to 0
         timeIsSync = false;
         askTime();
       }
      #endif
      saveMinute = minute();  // to test next minute
      utcTime = now();
      locTime = dst.localTime(utcTime, true);
      riseSet.calcSun(utcTime, dst.isItDst(utcTime, false));  // get sunrise/set and DST times
      /** start of a new day? **/
      if (saveDay != weekday(locTime)) saveDay = weekday(locTime);
      /** test for actions **/
      unitActions();
    }
  }
  #if !(defined(SET_AS_NODE_WS) || defined(USE_RTC_DC3231))
    else {
      /** ask first time sync Time? **/
      if(askTimer.tmrTime()) {
       timeIsSet = false;
       askTime();
      }
    }
  #endif
}
//======================================================================
#ifdef USE_ASK_NODE  
  //##### INSERT: all you need #####//
  /** ask Temp and Lux from NodeWS **/
  void ascTmpLux(const char *fromNode) {
    char str[bufLen + 1];
    sprintf(str, "|%1s|%1s|3|2| | | | | |0|", fromNode, nodeID);
    sendMessage(str);
  }
#endif

#if !(defined(SET_AS_NODE_WS) || defined(USE_RTC_DC3231))
  /** ask for Time from .... **/ 
  void askTime() {
    char str[bufLen + 1];
    #ifdef USE_DEBUG 
      printf("timeAskNode: %d, timeAskNodes: %s \n", timeAskNode, timeAskNodes[timeAskNode]);    
    #endif
    if (strcmp(nodeID, timeAskNodes[timeAskNode]) == 0) timeAskNode++;  // can not ask me self
    sprintf(str, "|%s|%1s|3|0| | | | | |0|", timeAskNodes[timeAskNode], nodeID);
    if (strcmp(timeAskNodes[timeAskNode], "99") == 0) timeAskNode = 0;
    else timeAskNode++;
    sendMessage(str);
  }
#endif

time_t syncProvider() {
  return tms.getTimestamp();
}

#ifndef SET_AS_NODE_WS
  /** node receive callback **/
  void receivedCallback(uint32_t from, String &msg) {
    #ifdef USE_DEBUG 
      printf("Node%s receivedCallback: %s\n", nodeID, msg.c_str());  
    #endif
    // "from" is not used, we use in the msg-string a node indentifier 
    strcpy((char *)nInBuf, msg.c_str());
    nodeMessage = true;
  }
#endif

/** put data in structure outBuf **/
void setOutBuf(char *_to, char *_from, char *_function, char *_command, char *_data0, char *_data1, char *_data2, char *_data3, char *_timestamp, char *_mode) {
  strcpy(outBuf.to,        _to);
  strcpy(outBuf.from,      _from);
  strcpy(outBuf.function,  _function);
  strcpy(outBuf.command,   _command);
  strcpy(outBuf.data[0],   _data0);
  strcpy(outBuf.data[1],   _data1);
  strcpy(outBuf.data[2],   _data2);
  strcpy(outBuf.data[3],   _data3);
  strcpy(outBuf.timestamp, _timestamp);
  strcpy(outBuf.mode,      _mode);
}

/** copy inBuf to outBuf, change 'to' with 'from', insert data-0-1-2 and set mode to "1" **/
void setInBufToOutBuf(char *data0) {
  setInBufToOutBuf(String(data0), space, space, space);
}
void setInBufToOutBuf(char *data0, char *data1) {
  setInBufToOutBuf(String(data0), String(data1), space, space);
}
void setInBufToOutBuf(char *data0, char *data1, char *data2) {
  setInBufToOutBuf(String(data0), String(data1), String(data2), space);
}
void setInBufToOutBuf(char *data0, char *data1, char *data2, char *data3) {
  setInBufToOutBuf(String(data0), String(data1), String(data2), String(data3));
}
void setInBufToOutBuf(String data0, String data1, String data2, String data3) {
  strcpy(outBuf.to,        inBuf.from);
  strcpy(outBuf.from,      inBuf.to);
  strcpy(outBuf.function,  inBuf.function);
  strcpy(outBuf.command,   inBuf.command);
  strcpy(outBuf.timestamp, inBuf.timestamp);  // return the same millis timestamp
  strcpy(outBuf.mode,      "1");              // 1: reply to a request
  strcpy(outBuf.data[0],   data0.c_str());    // insert data-0-1-2
  if(data1.length()) strcpy(outBuf.data[1], data1.c_str());
  else strcpy(outBuf.data[1], space);
  if(data2.length()) strcpy(outBuf.data[2], data2.c_str());
  else strcpy(outBuf.data[2], space);
  if(data3.length()) strcpy(outBuf.data[3], data3.c_str());
  else strcpy(outBuf.data[3], space);
}

/** put (split char string) data string in structure inBuf **/
void setInBuf(char *str) { 
  char s[bufLen + 1];
  strcpy (s, str);
  char *v, *b;
  v = strtok_r(s, "|", &b);
  strcpy(inBuf.to, v); 
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.from, v);
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.function, v);
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.command, v);
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.data[0], v);
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.data[1], v);
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.data[2], v);
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.data[3], v);
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.timestamp, v);
  v = strtok_r(NULL, "|", &b);
  strcpy(inBuf.mode, v);
}

/** put setOutBuf structure in a string **/
String setOutStr() {  // in caller: str = bufOut   or   str = a char string[bufLen + 1]
  char str[bufLen + 1];
  sprintf(str, "|%1s|%1s|%1s|%1s|%1s|%1s|%1s|%1s|%1s|%1s|", outBuf.to, outBuf.from, outBuf.function, outBuf.command, outBuf.data[0], outBuf.data[1], outBuf.data[2], outBuf.data[3], outBuf.timestamp, outBuf.mode);
  return String(str);
}

/** put data in a string **/
String setOutStr(char *_to, char *_from, char *_function, char *_command, char *_data0, char *_data1, char *_data2, char *_data3, char *_timestamp, char *_mode) {
  char str[bufLen + 1];
  sprintf(str, "|%1s|%1s|%1s|%1s|%1s|%1s|%1s|%1s|%1s|%1s|", _to, _from, _function, _command, _data0, _data1, _data2, _data3, _timestamp, _mode);
  return String(str);
}

/** get the millis after _start with overflow correction **/
uint32_t getMillis(uint32_t _start) {
  uint32_t _stop = millis();
  if (_stop < _start) return (uint32_t)4294967295 - _start + _stop + 1;
  else return _stop - _start;
}

/** get the pin, repetitions and duration **/
void setPin(boolean action) {
  uint8_t p;
  boolean f = action;
  if (atoi(shsDB.record[reversed]) != 0) f = !f;  // reverse action
  /** set the ON/DOWN or OFF/UP pin **/
  if (atoi(shsDB.record[pin2]) == -1) {
    p = pinToD(shsDB.record[pin1]);
    pinMode(p, OUTPUT);
    digitalWrite(p, f);
    strcpy(shsDB.record[pin], shsDB.record[pin1]);  // select the digital PIN
    strcpy(shsDB.record[status], (f ? "1" : "0"));  // ON/DOWN
    #ifdef USE_DEBUG
      printf("Pin %s: %d \n", shsDB.record[pin], f);
    #endif  
  }
  else {
    if (f) {
      p = pinToD(shsDB.record[pin1]);
      strcpy(shsDB.record[pin], shsDB.record[pin1]);  // select the digital PIN
      strcpy(shsDB.record[status], "1");  // ON/DOWN
    }
    else {
      p = pinToD(shsDB.record[pin2]);
      strcpy(shsDB.record[pin], shsDB.record[pin2]);  // select the digital PIN
      strcpy(shsDB.record[status], "0");  // ON/DOWN
    }
    /** momentary **/
    uint8_t r = atoi(shsDB.record[repetitions]);
    uint32_t ms = millis() , d = atoi(shsDB.record[duration]); 
    pinMode(p, OUTPUT);
    f = true;
    for (uint8_t i = 0; i < r; i++) {
      digitalWrite(p, f); 
      #ifdef USE_DEBUG
        printf("Pin GPIO%d/%s: %d \n", p, shsDB.record[pin], f);
      #endif  
      f = !f;
      while (getMillis(ms) < d) yield();
      ms = millis(); 
    }
  }
}

/** pin conversion from char to D-pin **/
uint8_t pinToD(char * p) {
  return pinD[atoi(String(p[1]).c_str())];
}

/** pin conversion from char to D-pin **/
uint8_t pinToA(char * p) {
  char pin[3];
  strcpy(pin, p + 1);
  #ifdef USE_ADC_ADS1115
    return atoi(pin);
  #else
    return pinA[atoi(pin)];
  #endif
}

#ifdef SET_AS_NODE_WS
  /** webSocket Event **/
  void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t len) {
  #ifdef USE_DEBUG
  printf("webSocketEvent in: %s\n", (char*)payload);
  #endif
    switch (type) {
      case WStype_DISCONNECTED: {
        break;
      }
      case WStype_CONNECTED: {
        char *p, *b;
        p = strtok_r((char*)payload + 1, "/", &b);
        if ((strcmp(p, socketName)) != 0) webSocket.disconnect(num);  // wrong socketName
        break;
      }
      case WStype_TEXT: { // msg from client #n
        if (payload[0] == 124 && payload[len - 1] == 124)  // 124 is char '|'
          strcpy(wsQueue[wsQueueItems++], (char*)payload);
        break;
      }
    }
  }
  
  /** softwareserial input from Node0 module **/
  void doReceiveSw() {
    while (swSer.available()) {
      char c = swSer.read();
      if (c < ' ' || c > '|') swBufIn[0] = '\0';
      else {
        if (c == '@') strcpy(swBufIn, "|");  // start of a new string
        else if (swBufIn[0] == '|') {  // "|" is a field separator and start char[0]
          if (c == '#') {  // end of the string
            strcat(swBufIn, "|");
            swMessage = true;
          }
          else strcat(swBufIn, String(c).c_str());
         }
       }
    }
  }

  /** serial input for my, browser/App or node(s)? **/
  void swReceived() {
    #ifdef USE_DEBUG
    printf("NodeWS swReceived: %s\n", swBufIn);  // demo code
    #endif
    /** data is excusive for my? **/
    if (strncmp(swBufIn + 1, nodeID, 2) == 0) nodeReceived(false);
    else {  /** data is also for my? **/
      if (strncmp(swBufIn, "|99|", 4) == 0) nodeReceived(false);
      /**  and/or other(s) **/
      String str = swBufIn;
      sendMessage(str);
    }
    swBufIn[0] = '\0';
  }

  void setupWS() {
    Serial.begin(115200);
//    Serial.setDebugOutput(true);
    swSer.begin(115200);
    /** WiWi start **/
    WiFi.disconnect();
    WiFi.mode(WIFI_STA);
    WiFi.config(local_IP, gateway_IP, subnet_IP);  // we use a static IP address and Port forwarding as server on the router
    WiFi.begin(mySSID, myPW);
    while (WiFi.waitForConnectResult() != WL_CONNECTED) delay(500);
    /** start websocket client **/
    webSocket.begin();
    webSocket.onEvent(webSocketEvent);
    /** sync the (library)Time with the (NTP)Time **/
    setSyncProvider(syncProvider);
    setSyncInterval(syncInterval);
    utcTime = now();
    locTime = dst.localTime(utcTime, true);  // get DST/Summertime times and isItDST
    /** calc several time data **/
    riseSet.calcSun(utcTime, dst.isItDst(utcTime, false));  // get sunrise/set and DST times
    saveMinute = minute();
    saveDay = weekday(locTime);
    timeIsSet = true;
  }
  
  void loopWS() {
    if (WiFi.status() != WL_CONNECTED) ESP.restart();
    webSocket.loop();
  
    /** websocket message? **/
    if (wsQueueItems) {  // a new ws message?
      strcpy((char *)nInBuf, wsQueue[0]);
      if (wsQueueItems > 1) for (uint8_t i = 1; i < wsQueueItems; i++) strcpy(wsQueue[i - 1], wsQueue[i]);
      wsQueueItems--;
      nodeReceived(true);
    }
    /** sofwareSerial chars? **/
    if (swSer.available()) doReceiveSw();
    /** sofwareSerial message? **/
    if (swMessage) {  // a new sw message?
      swMessage = false;
      swReceived();
    }
  }
#endif

#ifdef SET_AS_NODE_00
  /** softwareserial input from web module or browser/App **/
  void doReceiveSw() {
    while (swSer.available()) {
      char c = swSer.read();
      if (c < ' ' || c > '|') swBufIn[0] = '\0';
      else {
        if (c == '@') strcpy(swBufIn, "|");  // start of a new string
        else if (swBufIn[0] == '|') {  // "|" is a field separator and start char[0]
          if (c == '#') {  // end of the string
            strcat(swBufIn, "|");
            swMessage = true;
          }
          else strcat(swBufIn, String(c).c_str());
        }
       }
    }
  }
  
  /** serial input for my, a other or all? **/
  void swReceived() {
    #ifdef USE_DEBUG 
    printf("Node00 swReceived: %s\n", swBufIn);  
    #endif
    /** data is excusive for my? **/
    if (strncmp(swBufIn + 1, nodeID, 2) == 0) nodeReceived(false);
    else {  /** data is also for my? **/
      if (strncmp(swBufIn, "|99|", 4) == 0) nodeReceived(false);
      /**  and/or other(s) **/
      String str = String(swBufIn);
      mesh.sendBroadcast(str);
    }
    swBufIn[0] = '\0';
  }

  void setup00() {
    Serial.begin(115200);
  //  Serial.setDebugOutput(true); 
    swSer.begin(115200);
  }
  void loop00() {
    if (nodeMessage) {
      nodeMessage = false;
      nodeReceived(true);  // a new node message?
    }
    if (swSer.available()) doReceiveSw();
    if (swMessage) {  // a new sw message?
      swMessage = false;
      swReceived();
    }
  }
#endif


#ifdef USE_PUBLISH
  /** switch LEDs **/
  void setPublishLED() {
    if (buttonState == 1) {
      flashPin = redPin;
      digitalWrite(greenPin, HIGH);  // OFF!!!
      digitalWrite(redPin, LOW);     // ON!!!
    }
    else {
      flashPin = greenPin;
      digitalWrite(redPin, HIGH);   // OFF!!!
      digitalWrite(greenPin, LOW);  // ON!!!
    }
  }
  
  void sendPublish() {
    /** send commands **/
    char str[bufLen + 1];
    time_t d = millis();
    for (uint8_t i = 0; i < nNodes; i++) {
      if (buttonState) {  // "going away"
        sprintf(str, "|%s|%s|%s|%s|%s|%s|%s|%s|0|0|", 
          onPublish[i][0], nodeID, onPublish[i][1], onPublish[i][2],
          onPublish[i][3], onPublish[i][4], onPublish[i][5], onPublish[i][6]);
      }
      else {
        sprintf(str, "|%s|%s|%s|%s|%s|%s|%s|%s|0|0|",
          offPublish[i][0], nodeID, offPublish[i][1], offPublish[i][2],
          offPublish[i][3], offPublish[i][4], offPublish[i][5], offPublish[i][6]);
      }
      sendMessage(str);
    }
  }
  
  /** interrupt from the command switch **/
  void switchPublish() {
    if (getMillis(msIntr) >= msDebouncePublish) {  // msec. debounce interrupt trigger
      #ifdef USE_DEBUG
        printf("Node%s Publish button 2\n", nodeID);  
      #endif
      buttonState = buttonState != 0 ? 0 : 1;   // reverse state
      shsDB.readRec(publishKey);
      strcpy(shsDB.record[status], (buttonState ? "1" : "0"));
      shsDB.writeRec(publishKey);
      sendPublish();
      setPublishLED();  // set LEDs
      msIntr = millis();  // new debounce time
    }
  }

  void setupPublish() {
    pinMode(greenPin, OUTPUT);
    pinMode(  redPin, OUTPUT);
    digitalWrite(greenPin, HIGH);  // green OFF, active LOW
    digitalWrite(  redPin, HIGH);   //   red OFF, active LOW
    attachInterrupt(digitalPinToInterrupt(pinToD(publishPin)), switchPublish, FALLING);
    /** The Publish button record key: "PuBl" **/
    if (shsDB.db.findRec(publishKey) == NOT_FOUND) {  // if not exist? create it
      shsDB.newRecord();  // get the default values in record[]
      strcpy(shsDB.record[pin1], publishPin);
      strcpy(shsDB.record[pin2], publishPin);
      strcpy(shsDB.record[repetitions], "2");
      strcpy(shsDB.record[duration], "0");
      shsDB.writeRec(publishKey);
    }
  }
  
  void loopPublish() {  
    if (flash.tmrTime()) {
      flash.tmrReset((ledON ? onTimePublish : offTimePublish), true);
      ledON = !ledON;  // LOW active!!!
      digitalWrite(flashPin, ledON);
    }
  }
#endif  

