/** simpleHomeSwitch    **
 *  www.kendziorra.nl    *
 *  Ewald sr, 2018       *
 *  totally free to use  *
 *************************
 * automatic actions code
 *************************/

#include "miscellany.h"

/** test for automatic mode actions **/
void unitActions() {
  #ifdef USE_DEBUG 
    printf("@@@ ESP free heap size: %u\n", ESP.getFreeHeap());
    printf("unitActions db.dbR: %d, db.dbT: %d \n", shsDB.db.dbR, shsDB.db.dbT);
    printf("onAction: %02u:%02u \n", hour(locTime), minute(locTime));
  #endif
  if (!shsDB.db.dbR) return;  // no records no action
  /** test all the units (records) for action **/
  for (uint8_t r = 0; r < shsDB.db.dbT; r++) {
    shsDB.getRec(r);
    #ifdef USE_DEBUG 
      printf("onAction: %u, recKey: %s-%s \n", r, shsDB.recKey, shsDB.db.dbB);
    #endif   
    if (shsDB.db.dbResult == TOMBSTONE) continue;  // no action, skip this record
    if (shsDB.db.dbResult != DB_OK) break;  // exit for loop?
    /** stop or manual mode? **/
    if (shsDB.record[pause][0] == '2' || shsDB.record[mode][0] == '0') continue;  // no action, skip this record
    /** 04:00am and paused? reset pause? **/
    if (shsDB.record[pause][0] == '1') {
      if (hour(locTime) == 4 && minute(locTime) <= 5) {
        strcpy(shsDB.record[pause], "0");
        shsDB.writeRec(shsDB.recKey);  
      }
      continue;  // next record
    }
    /** action? **/
    uint8_t action = NO;
    #ifdef USE_LUX_BH1750 
      luminance = Lux.getLux();
    #endif
    #ifdef USE_TEMP_DS18B20
      double t = Temp1.getTemp();
      temperature = (!fahrenheit ? t : t * 1.8 + 32.0);  // !fahrenheit ? celsius : fahrenheit
    #elif defined(USE_THERMISTOR)
      #ifdef USE_ADC_ADS1115
        adcSteps = adc.getAdc(adcPin);
      #else
        adcSteps = analogRead(adcPin);
      #endif
      double t = therm.getTempC(adcSteps);
      temperature = (!fahrenheit ? t : t * 1.8 + 32.0);  // !fahrenheit ? celsius : fahrenheit
    #endif
    switch (atoi(shsDB.record[mode])) {       // mode    : ON/DOWN             OFF/UP
      // -----------------------------------------
      case  1: action = doTime();     break;  // Time    : time 1               time 2
      case  2: action = doLux();      break;  // Lux     : lux  1               lux  2
      case  3: action = doTemp();     break;  // Temp    : temp 1               temp 2
      case  4: action = doRise();     break;  // Rise    : time 1               sunRise + offset 1
      case  5: action = doSet();      break;  // Set     : sunSet  - offset 2   time 2
      case  6: action = doSun();      break;  // Sun     : lux  1 && temp   1   lux  2 && temp   2 || yoyo
      case  7: action = doDay();      break;  // Day     : sunRise - offset 1   sunSet  + offset 2
      case  8: action = doNight();    break;  // Night   : sunSet  - offset 1   sunRise + offset 2
      case  9: action = doTimer();    break;  // Timer   : manual               after counter1-minutes
      case 10: action = doTimeTemp(); break;  // TimeTemp: time 1 && temp   1   time 2 || temp   2 
    }
    #ifdef USE_DEBUG
      printf("unitActions acction: %s \n", (action == NO ? "NO" : action == ON ? "ON" : action == OFF ? "OFF" : "SAVE"));
    #endif
    /** apply action? **/
    if (action != NO) {
      if (action != SAVE) {
        boolean publish = false;
        #ifdef USE_PUBLISH
          if(strcmp(shsDB.record[pin1], publishPin) == 0) {  // Publish button pin?
            publish = true;
            strcpy(shsDB.record[status], (action == OFF ? "0" : "1"));
            buttonState = (action == OFF ? 0 : 1); 
            sendPublish();
            setPublishLED();  // set LEDs
          }
        #endif
        if (!publish) {
          /** is it a StepperSteps unit? **/
          if (strcmp(shsDB.record[pin], "stps") == 0) {
            #ifdef USE_STEPS
              /** it's a StepperSteps unit **/
              uint8_t r = atoi(shsDB.record[pin2]);
              if (action == OFF) stepper.gotoHome();   // find home position (using the maximun number of rotations)
              else if (action == ON)  {
                if (r == 0) stepper.gotoEnd();  // find end position (using the maximun number of rotations)
                else stepper.stepTo(r);  //  go to step nn
              }
              strcpy(shsDB.record[status], (action == ON ? "1" : "0"));
            #endif
          }
          #ifdef USE_DEGREES
             /** is it a DegreesStep unit? **/
            else if (strcmp(shsDB.record[pin], "dgrs") == 0) {
              /** it's a DegreesStep unit **/
              if (action == OFF) degr.findHome();   // find home position (using the maximun number of rotations)
              else degr.toDegree(atoi(shsDB.record[pin2]));  // do nn steps
              strcpy(shsDB.record[status], (action == ON ? "1" : "0"));
            }
          #endif
          /** kaku unit? **/
          else if (strlen(shsDB.record[period]) == 1) {  // a KaKu period has a length of 3
            /** it's not a KaKu unit **/
            setPin((action == OFF ? false : true));
          }
          #ifdef USE_KAKU
            else {
              /** it's a KaKu unit **/
              char dim[] = "0";  // pre set OFF/UP
              if (action == ON) dim[0] = (atoi(shsDB.record[dimLevelPresent]) == 0 ? '1' : '2');
              strcpy(shsDB.record[status], dim);  // save status
              if (strlen(shsDB.record[code]) > 1) kaku.kakuOld(shsDB.record[reversed], shsDB.record[status], shsDB.record[period], shsDB.record[code]);   // KaKu New has no code
              else kaku.kakuNew(shsDB.record[reversed], shsDB.record[status], shsDB.record[period], shsDB.record[address], shsDB.record[dimLevelPresent], shsDB.record[dimLevel], shsDB.record[unit], shsDB.record[groupBit]);
            }
          #endif
        }
      }
      shsDB.writeRec(shsDB.recKey);  // save record
    }
  }
}

/** simple ON/DOWN or OFF/UP on time **/
uint8_t doTime() {
  return actionTime(atoi(shsDB.record[hh1]), atoi(shsDB.record[mm1]), 0, atoi(shsDB.record[hh2]), atoi(shsDB.record[mm2]), 0);
}

/** unit ON/DOWN or OFF/UP on lux **/
uint8_t doLux() {
  uint8_t result = OFF;
  #ifdef USE_LUX_BH1750
    uint8_t _yoyo = atoi(shsDB.record[yoyo]);
    int8_t _counter1 = atoi(shsDB.record[counter1]), _counter2 = atoi(shsDB.record[counter2]), _counter3 = atoi(shsDB.record[counter3]);
    uint16_t l1 = atoi(shsDB.record[lux1]), l2 = atoi(shsDB.record[lux2]);
    boolean l = l1 > l2;  // lux1 > lux2: false, lux1 < lux2: true (used for dead zone and OFF/UP test)
    /** Temp and Lux are set? **/
    if (luminance == 999999) {
      _counter1 = -1;
      _counter2 = -1;
      _counter3 = -1;
      result = OFF;  // wait for Temp and Lux
    }
    /** dead luminance zone, reset al the counters **/
    if ((l && (luminance > l2 && luminance < l1)) || (!l && (luminance > l1 && luminance < l2))) {
      if (_counter1 != -1 || _counter2 != -1 || _counter3 != -1) {
        _counter1 = _counter2 = _counter3 = -1;  // reset counters
        result = SAVE;
      }
    }
    /** set yoyo counters **/
    else if (atoi(shsDB.record[status]) == 0) {  // unit is OFF/UP
  
      /** unit switch ON/DOWN? **/
      if (luminance >= l1) {
        _counter1++;
        _counter2 = _counter3 = -1;  // reset counters
        result = SAVE;
        if (_counter1 >= _yoyo) {
          _counter1 = -1;  // reset ON/DOWN yoyo
          result = ON;
        }
      }
    }
    else {  // unit is ON/DOWN
      /** unit switch UP on bad-weather? **/
      if (l && (luminance <= atoi(shsDB.record[badLux]))) {
        _counter2++;
        _counter1 = _counter3 = -1;  // reset counters
        result = SAVE;
        testTime = now() - (60 - badWeatherSec);  // next measurement after n sec. (default = 20 sec.)
        if (_counter2 >= 1) {  // after two measurements in 20 sec.
          _counter2 = -1;  // reset OFF/UP bad
          result = OFF;
        }
      }
      /** unit switch OFF/UP? **/
      else if ((l && (luminance <= l2)) || (!l && (luminance >= l2))) {
        _counter3++;
        _counter1 = _counter2 = -1;  // reset counters
        result = SAVE;
        if (_counter3 >= _yoyo) {
          _counter3 = -1;  // reset OFF/UP yoyo
          result = OFF;
        }
      }
    }
    strcpy(shsDB.record[counter1], String(_counter1).c_str());
    strcpy(shsDB.record[counter2], String(_counter2).c_str());
    strcpy(shsDB.record[counter3], String(_counter3).c_str());
    /** record is updated in unitActions() **/
  #endif
  return result;
}

/** unit ON/DOWN or OFF on temp **/
uint8_t doTemp() {
  uint8_t result = OFF;
  #ifdef USE_TEMP_DS18B20
    uint8_t _yoyo = atoi(shsDB.record[yoyo]);
    int8_t _counter1 = atoi(shsDB.record[counter1]), _counter2 = atoi(shsDB.record[counter3]);
    uint16_t t1 = atoi(shsDB.record[tmp1]), t2 = atoi(shsDB.record[tmp2]);
    boolean t = t1 > t2;  // tmp1 > tmp2: false, tmp1 < tmp2: true (used for dead zone and OFF/UP test)
    /** Temp and Lux are set? **/
    if (temperature == -999) {
      _counter1 = -1;
      _counter2 = -1;
      result = OFF;  // wait for Temp and Lux
    }
    /** dead temperature zone, reset al the counters **/
    else if ((t && (temperature > t2 && temperature < t1)) || (!t && (temperature > t1 && temperature < t2))) {
      if (_counter1 != -1 || _counter2 != -1) {
        _counter1 = _counter2 =  -1;  // reset counters
        result = SAVE;
      }
    }
    else if (atoi(shsDB.record[status]) == 0) {  // unit is OFF/UP
      /** unit switch ON/DOWN? **/
      if (temperature >= t1) {
        _counter1++;
        _counter2 = -1;  // reset OFF/UP yoyo
        result = SAVE;
        if (_counter1 >= _yoyo) {
          _counter1 = -1;  // reset ON/DOWN yoyo
          result = ON;
        }
      }
    }
    else {  // unit is ON/DOWN
      /** unit switch OFF/UP? **/
      if ((t && (temperature <= t2)) || (!t && (temperature >= t2))) {
        _counter2++;
        _counter1 = -1;  // reset ON/DOWN yoyo
        result = SAVE;
        if (_counter2 >= _yoyo) {
          _counter2 = -1;  // reset OFF/UP yoyo
          result = OFF;
        }
      }
    }
    strcpy(shsDB.record[counter1], String(_counter1).c_str());
    strcpy(shsDB.record[counter3], String(_counter2).c_str());
    /** record is updated in unitActions() **/
  #endif
  return result;
}

/** ON/DOWN on time, OFF/UP on sunRise + rise offset **/
uint8_t doRise() {
  return actionTime(atoi(shsDB.record[hh1]), atoi(shsDB.record[mm1]), 0, riseSet.sunRiseHour, riseSet.sunRiseMinute, atoi(shsDB.record[riseOffset]));
}

/** ON/DOWN on sunSet - set offset, OFF/UP on time **/
uint8_t doSet() {
  return actionTime(riseSet.sunSetHour, riseSet.sunSetMinute, -atoi(shsDB.record[setOffset]), atoi(shsDB.record[hh2]), atoi(shsDB.record[mm2]), 0);
}

/** unit DOWN or UP on temp and lux (my sun screen) - with anti yo-yo and bad weather mode **/
uint8_t doSun() {
  uint8_t result = NO, _yoyo = atoi(shsDB.record[yoyo]);
  int8_t _counter1 = atoi(shsDB.record[counter1]), _counter2 = atoi(shsDB.record[counter2]), _counter3 = atoi(shsDB.record[counter3]);
  uint16_t l1 = atoi(shsDB.record[lux1]), l2 = atoi(shsDB.record[lux2]), t1 = atoi(shsDB.record[tmp1]), t2 = atoi(shsDB.record[tmp2]);
  /** unit is UP od DOWN? **/
  /** dead luminance/temperature zone, reset al the counters **/
  if ((luminance > l2 && luminance < l1) ||
      (temperature > t2 && temperature < t1)) {
    if (_counter1 != -1 || _counter2 != -1 || _counter3 != -1) {
      _counter1 = _counter2 = _counter3 = -1;  // reset counters
      result = SAVE;
    }
  }
  else if (atoi(shsDB.record[status]) == 0) {  // unit is UP
    /** unit switch DOWN? **/
    if (temperature >= t1 && luminance >= l1) {
      _counter1++;
      _counter2 = -1;  // reset OFF/UP bad
      _counter3 = -1;  // reset OFF/UP yoyo
      result = SAVE;
      if (_counter1 >= _yoyo) {
        _counter1 = -1;  // reset ON/DOWN yoyo
        result = ON;
      }
    }
  }
  else {  // unit is down
    /** unit switch UP on bad-weather? **/
    if (luminance <= atoi(shsDB.record[badLux])) {
      _counter3++;
      _counter1 = -1;  // reset ON/DOWN yoyo
      _counter2 = -1;  // reset OFF/UP yoyo
      result = SAVE;
      if (_counter3 >= 1) {
        _counter3 = -1;  // reset OFF/UP bad
        result = OFF;
      }
    }
    /** unit switch UP on low temp or low lux? **/
    else if (temperature <= t2 || luminance <= l2) {
      _counter2++;
      _counter1 = -1;  // reset ON/DOWN yoyo
      _counter3 = -1;  // reset OFF/UP bad
      result = SAVE;
      if (_counter2 >= _yoyo) {
        _counter2 = -1;  // reset OFF/UP yoyo
        result = OFF;
      }
    }
  }
  strcpy(shsDB.record[counter1], String(_counter1).c_str());
  strcpy(shsDB.record[counter2], String(_counter2).c_str());
  strcpy(shsDB.record[counter3], String(_counter3).c_str());
  /** record is updated in unitActions() **/
  return result;
}

/** ON on sunRise - offset, OFF on sunSet + offset **/
uint8_t doDay() {
  return actionTime(riseSet.sunRiseHour, riseSet.sunRiseMinute, -atoi(shsDB.record[riseOffset]), riseSet.sunSetHour,  riseSet.sunSetMinute, atoi(shsDB.record[setOffset]));
}

/** OFF on sunRise + rise offset, ON on sunSet - set offset **/
uint8_t doNight() {
  return actionTime(riseSet.sunSetHour, riseSet.sunSetMinute, -atoi(shsDB.record[setOffset]), riseSet.sunRiseHour, riseSet.sunRiseMinute, atoi(shsDB.record[riseOffset]));
}

/** manual ON/DOWN, OFF/UP after counter1-minutes **/
uint8_t doTimer() {
  uint8_t result = NO;
  int16_t _counter1 = atoi(shsDB.record[counter1]), _counter2 = atoi(shsDB.record[counter2]);
  /** unit is ON/DOWN? **/
  if (atoi(shsDB.record[status]) != 0) {  // unit is ON/DOWN
    /** unit switch OFF/UP? **/
    _counter2++;
    result = SAVE;
    if (_counter2 >= _counter1) {
      _counter2 = -1;  // reset counter
      result = OFF;
    }
  }
  strcpy(shsDB.record[counter2], String(_counter2).c_str());
  /** record is updated in unitActions() **/
  return result;
}

/** test ON/OFF time and temperature **/
uint8_t doTimeTemp() {
  uint8_t result = OFF;
  #if defined(USE_TEMP_DS18B20) || defined(USE_THERMISTOR)
    if (temperature == -999) result = OFF; 
    else {
      time_t t0 = dst.calcTimeStamp(hour(locTime), minute(locTime), 0, 1, 1, 1970);
      time_t t1 = dst.calcTimeStamp(atoi(shsDB.record[hh1]), atoi(shsDB.record[mm1]), 0, 1, 1, 1970);
      time_t t2 = dst.calcTimeStamp(atoi(shsDB.record[hh2]), atoi(shsDB.record[mm2]), 0, 1, 1, 1970);
      if (t2 < t1) {                          // between 24:00 between 04:00 ?
        t2 += dst.oneDay;                     // add 24h
        if (t0 < 4) t0 += dst.oneDay;         // add 24h
      }
      if (t0 >= t1 && t0 < t2) {
        //@@@@@@@@@@@ NodeID "04" boiler specific tmperature control @@@@@@@@@@@//
        if (strcmp(shsDB.recKey, "tmp1") == 0) {
          uint8_t _tmp1 = 48, _tmp2 = 50;
          uint16_t _time = hour(locTime) * 60 + minute(locTime);
          if ((_time >= ( 9 * 60 +  0) &&  _time < (12 * 60 + 30)) || 
              (_time >= (16 * 60 + 30) &&  _time < (19 * 60 + 30))) {
            _tmp1 = 76;
            _tmp2 = 78;
          }
          #ifdef USE_DEBUG
            printf("Node%s BoilerTemp: %s, temp ON: %d, temp OFF: %d \n", nodeID, String(temperature).c_str(), _tmp1, _tmp2);  
          #endif
          if (temperature >= _tmp2) result = OFF;
          else if (temperature <= _tmp1) result = ON;
        }
        else { //@@@@@@@@@@@//
          if (temperature >= atoi(shsDB.record[tmp2])) result = OFF;
          else if (temperature <= atoi(shsDB.record[tmp1])) result = ON;
        }     //@@@@@@@@@@@//
      }
    }
  #endif
  return result;
}

/** test ON/OFF time **/
uint8_t actionTime(const uint8_t h1, const uint8_t m1, const int8_t o1, const uint8_t h2, const uint8_t m2, const int8_t o2) {
  time_t t0 = dst.calcTimeStamp(hour(locTime), minute(locTime), 0, 1, 1, 1970);
  time_t t1 = dst.calcTimeStamp(h1, m1, 0, 1, 1, 1970) + o1 * 60;
  time_t t2 = dst.calcTimeStamp(h2, m2, 0, 1, 1, 1970) + o2 * 60;
  if (t0 == t1 || (t0 - 1) == t1) return ON;  // switch only on ON/DOWN minutes
  else if (t0 == t2 || (t0 + 1) == t2) return OFF;  // switch only on OFF/UP  
  return NO;  // no action
}
