/** simpleHomeSwitch    **
 *  www.kendziorra.nl    *
 *  Ewald sr, 2018       *
 *  totally free to use  *
 *************************
 * (re)start actions code
 *************************/

#include "miscellany.h"

/** set status on (re)start **/
void unitStart() {
  doStart = false;       // not from loop()
  #ifdef USE_PUBLISH
    /** (re)set button pin **/
    pinMode(pinToD(publishPin), 2);
  #endif
  #ifdef USE_DEBUG 
    printf("unitStart db.dbR: %d, db.dbT: %d \n", shsDB.db.dbR, shsDB.db.dbT);
  #endif
  // only Kaku and Pin units are, depended on mode, set to ON/OFF 
  // tested modes: 1:Time, 2:Lux, 3:Temp, 4:sunRise, 5:sunSet, 7:Day, 8:Night
  // mode 6:sun (combined Temp and Lux) will set by unitActions()
  /** test all the units (records) for start status **/
  if (!shsDB.db.dbR) return;  // no records no action
  for (uint8_t r = 0; r < shsDB.db.dbT; r++) {
    shsDB.getRec(r);
    #ifdef USE_DEBUG 
      printf("onStart: %u, recKey: %s-%s \n", r, shsDB.recKey, shsDB.db.dbB);
    #endif   
    /** stop or manual mode? **/
    if (shsDB.record[pause][0] == '2') continue;  // no action, skip this record
    /** 04:00am and paused? reset pause? **/
    if (shsDB.record[pause][0] == '1' && hour(locTime) == 4 && minute(locTime) <= 5) {
      strcpy(shsDB.record[pause], "0");
      shsDB.writeRec(shsDB.recKey);
      continue;  // next record
    }
    /** action? **/
    uint8_t action = NO;
    #ifdef USE_LUX_BH1750 
      luminance = Lux.getLux();
    #endif
    #ifdef USE_TEMP_DS18B20
      double t = Temp1.getTemp();
      temperature = (!fahrenheit ? t : t * 1.8 + 32.0);  // !fahrenheit ? celsius : fahrenheit
    #elif defined(USE_THERMISTOR)
      #ifdef USE_ADC_ADS1115
        adcSteps = adc.getAdc(adcPin);
      #else
        adcSteps = analogRead(adcPin);
      #endif
      double t = therm.getTempC(adcSteps);
      temperature = (!fahrenheit ? t : t * 1.8 + 32.0);  // !fahrenheit ? celsius : fahrenheit
    #endif
    switch (atoi(shsDB.record[mode])) {            // mode :    ON/DOWN                     OFF/UP
      // -----------------------------------------
      case  0: action = doRepeat();        break;  // repeat the last manual status
      case  1: action = doStartTime();     break;  // Time :    time 1                      time 2
      case  2: action = doStartLux();      break;  // Lux  :    lux  1                      lux  2
      case  3: action = doStartTemp();     break;  // Temp :    temp 1                      temp 2
      case  4: action = doStartRise();     break;  // Rise :    time 1                      sunRise + offset 1
      case  5: action = doStartSet();      break;  // Set  :    sunSet  - offset 2          time 2
      case  6: action = NO;                break;  // Sun  :    lux  1 && temp   1 && yoyo  lux  2 && temp    2 && yoyo
      case  7: action = doStartDay();      break;  // Day  :    sunRise - offset 1          sunSet  + offset  2
      case  8: action = doStartNight();    break;  // Night:    sunSet  - offset 1          sunRise + offset  2
      case  9: action = NO;                break;  // Timer:    manual                      after counter1-minutes
      case 10: action = doStartTimeTemp(); break;  // TimeTemp: time 1 && temp   1          time 2 || temp   2 
    }
    #ifdef USE_DEBUG
      printf("unitStart acction: %s \n", (action == NO ? "NO" : action == ON ? "ON" : action == OFF ? "OFF" : "SAVE"));
    #endif
    /** apply action? **/
    if (action != NO) {
      boolean publish = false;
      #ifdef USE_PUBLISH
        if(strcmp(shsDB.record[pin1], publishPin) == 0) {  // Publish button pin?
          publish = true;
          buttonState = (atoi(shsDB.record[status]) != 0 ? 1 : 0); 
          sendPublish();
          setPublishLED();  // set LEDs
        }
      #endif
      if (!publish) {
        /** is it a StepperSteps unit? **/
        if (strcmp(shsDB.record[pin], "stps") == 0) {
          #ifdef USE_STEPS
            /** it's a StepperSteps unit **/
            uint8_t r = atoi(shsDB.record[pin2]);
            if (action == OFF) stepper.gotoHome();   // find home position (using the maximun number of rotations)
            else if (action == ON)  {
              if (r == 0) stepper.gotoEnd();  // find end position (using the maximun number of rotations)
              else stepper.stepTo(r); //  go to step nn
            }
            strcpy(shsDB.record[status], (action == ON ? "1" : "0"));
          #endif
        }
        #ifdef USE_DEGREES
           /** is it a DegreesStep unit? **/
          else if (strcmp(shsDB.record[pin], "dgrs") == 0) {
            /** it's a DegreesStep unit **/
            if (action == OFF) degr.findHome();   // find home position (using the maximun number of rotations)
            else degr.toDegree(atoi(shsDB.record[pin2]));  // do nn steps
            strcpy(shsDB.record[status], (action == ON ? "1" : "0"));
          }
        #endif
        /** kaku unit? **/
        else if (strlen(shsDB.record[period]) == 1) {  // a KaKu period has a length of 3
          /** it's not a KaKu unit **/
          setPin((action == OFF ? false : true));
        }
        #ifdef USE_KAKU
          else {
            /** it's a KaKu unit **/
            char dim[] = "0";  // pre set OFF/UP
            if (action == ON) dim[0] = (atoi(shsDB.record[dimLevelPresent]) == 0 ? '1' : '2');
            strcpy(shsDB.record[status], dim);  // save status
            if (strlen(shsDB.record[code]) > 1) kaku.kakuOld(shsDB.record[reversed], shsDB.record[status], shsDB.record[period], shsDB.record[code]);  // KaKu New has no code
            else kaku.kakuNew(shsDB.record[reversed], shsDB.record[status], shsDB.record[period], shsDB.record[address], shsDB.record[dimLevelPresent], shsDB.record[dimLevel], shsDB.record[unit], shsDB.record[groupBit]);
          }
        #endif
      }
      shsDB.writeRec(shsDB.recKey);  // save record
    }
  }
}

/** simple ON/DOWN or OFF/UP on time **/                   // mode : ON/DOWN             OFF/UP
                                                           // =================================
uint8_t doRepeat() {                                       // manual:repeat the last manual status
  if (atoi(shsDB.record[status]) == 0) return OFF;
  else return ON;
}

uint8_t doStartTime() {                                    // Time : time 1              time 2
  return actionStartTime(atoi(shsDB.record[hh1]), atoi(shsDB.record[mm1]), 0, atoi(shsDB.record[hh2]), atoi(shsDB.record[mm2]), 0, true);
}

/** unit ON/DOWN or OFF/UP on lux **/
uint8_t doStartLux() {                                     // Lux  : lux  1               lux  2
  uint8_t result = OFF;
  #ifdef USE_LUX_BH1750
    /** Temp and Lux are set? **/
    if (luminance == 999999) result = OFF; 
    if (luminance >= atoi(shsDB.record[lux1])) result = ON;
    if (luminance <= atoi(shsDB.record[lux2])) result = OFF;  
  #endif
  return result;
}

/** unit ON/DOWN or OFF on temp **/
uint8_t doStartTemp() {  
  uint8_t result = OFF;
  /** Temp and Lux are set? **/
  if (temperature == -999 || temperature >= atoi(shsDB.record[tmp2])) result = OFF;
  else if (temperature <= atoi(shsDB.record[tmp1])) result = ON;  
  return result;
}

/** ON/DOWN on time, OFF/UP on sunRise + rise offset **/
uint8_t doStartRise() {                                    // Rise : time 1               sunRise + offset 1
  return actionStartTime(atoi(shsDB.record[hh1]), atoi(shsDB.record[mm1]), 0, riseSet.sunRiseHour, riseSet.sunRiseMinute, atoi(shsDB.record[riseOffset]), true);
}

/** ON/DOWN on sunSet - set offset, OFF/UP on time **/
uint8_t doStartSet() {                                     // Set  : sunSet  - offset 2   time 2
  return actionStartTime(riseSet.sunSetHour, riseSet.sunSetMinute, -atoi(shsDB.record[setOffset]), atoi(shsDB.record[hh2]), atoi(shsDB.record[mm2]), 0, true);
}

/** ON on sunRise - offset, OFF on sunSet + offset **/
uint8_t doStartDay() {                                     // Day  : sunRise - offset 1   sunSet  + offset  2
  return actionStartTime(riseSet.sunRiseHour, riseSet.sunRiseMinute, -atoi(shsDB.record[riseOffset]), riseSet.sunSetHour,  riseSet.sunSetMinute, atoi(shsDB.record[setOffset]), true);
}

/** OFF on sunRise + rise offset, ON on sunSet - set offset **/
uint8_t doStartNight() {                                   // Night: sunSet  - offset 1   sunRise + offset  2
  return actionStartTime(riseSet.sunSetHour, riseSet.sunSetMinute, -atoi(shsDB.record[setOffset]), riseSet.sunRiseHour, riseSet.sunRiseMinute, atoi(shsDB.record[riseOffset]), false); 
}

/** test ON/OFF time and temperature **/
uint8_t doStartTimeTemp() {
  uint8_t result = OFF;
  #if defined(USE_TEMP_DS18B20) || defined(USE_THERMISTOR)
    if (temperature == -999) result = OFF;
    else {
      time_t t0 = dst.calcTimeStamp(hour(locTime), minute(locTime), 0, 1, 1, 1970);
      time_t t1 = dst.calcTimeStamp(atoi(shsDB.record[hh1]), atoi(shsDB.record[mm1]), 0, 1, 1, 1970);
      time_t t2 = dst.calcTimeStamp(atoi(shsDB.record[hh2]), atoi(shsDB.record[mm2]), 0, 1, 1, 1970);
      if (t2 < t1) {                          // between 24:00 between 04:00 ?
        t2 += dst.oneDay;                     // add 24h
        if (t0 < 4) t0 += dst.oneDay;         // add 24h
      }
      if (t0 >= t1 && t0 < t2) result = doStartTemp();
    }
  #endif
  return result;
}

/** test ON/OFF time **/
//##### a new day starts at 04:00:00 and ends at 03:59:59 #####//
uint8_t actionStartTime(const uint8_t h1, const uint8_t m1, const int8_t o1, const uint8_t h2, const uint8_t m2, const int8_t o2, boolean d) {
  time_t t0 = dst.calcTimeStamp(hour(locTime), minute(locTime), 0, 1, 1, 1970);
  time_t t1 = dst.calcTimeStamp(h1, m1, 0, 1, 1, 1970) + o1 * 60;
  time_t t2 = dst.calcTimeStamp(h2, m2, 0, 1, 1, 1970) + o2 * 60;
  if (d) {
    if (t2 < t1) {                          // between 24:00 between 04:00 ?
      t2 += dst.oneDay;                     // add 24h
      if (t0 < 4) t0 += dst.oneDay;         // add 24h
    }
    if (t0 >= t1 && t0 < t2) return ON;     // morning, day and evening
  }                                         // =======  ===     ======= mode
  else if (t0 < t2 || t0 >= t1) return ON;  // night
  return OFF;                               // ===== mode
}

