/** simpleHomeSwitch    **
 *  www.kendziorra.nl    *
 *  Ewald sr, 2018       *
 *  totally free to use  *
 *************************
 * miscellany code
 *************************/
 
#ifndef MICELLANI_H
  #define MICELLANI_H

  #include <simpleTimers.h>        // A simple alternative milliseconds timer(s) library - http://kendziorra.nl
  #include "EditMe.h"
  
  /** Mesh login data and indentification **/
  const char nodeSSID[] = "simpleHomeSwitch",  // the same SSID for all Nodes
               nodePW[] = "simpleHomeNode";    // the same PW for all Nodes
  const uint8_t nToLen = 2,        // (Node)ID  "00"-"nn" or: "WS" Server, "WB" Web Browser
                nFromLen = 2,      // (Node)ID 
                nfuncLen = 2,      // funtion to/from Node(n)
                nCmdLen = 2,       // command to Node(n)
                nDataLen = 10,     // 4 x data to/from Node(n)
                nTstampLen = 13,   // timestamp to/from Node(n)
                nModeLen = 1,      // set to: 0: command/request, 1: response
                bufLen = 
                   nToLen +      
                   nFromLen +
                   nfuncLen + 
                   nCmdLen +
                   nDataLen * 4 +
                   nTstampLen +
                   nModeLen + 11;  // total len + 11 field delimiters "|" (total 71)
  char bufOut[bufLen + 1],         // node out buffer
       bufIn[bufLen + 1];          // node in buffer
  /** used in callback function **/
  volatile char nInBuf[bufLen + 1];       // node in buffer
  volatile boolean nodeMessage =  false;  // if true: node message received
  double temperature = -999;      // holt the temperature
  uint32_t luminance = 999999;    // holt the luminance
  time_t utcTime,
         locTime, 
         testTime;               // used with uint8_t badWeatherSec  
  int16_t syncInterval = 1800;   // sync the time interval in seconds
  uint8_t saveMinute = 255,      // action time? test
          saveDay = 255,         // summetime/DST test
          badWeatherSec = 20,    // interval between two Lux measurements in sec.
          syncMinutes = 30,      // used if no RTC
          syncCounter = syncMinutes -1;  // used if no RTC
  char space[] = " ";            // used as withe space in data fields
  boolean timeIsSet = false,     // true: time is set
          doStart = true,        // unitStart() if time is sync
          timeIsSync = false;    // true: time is sync
  #ifdef SET_AS_NODE_WS
    const uint8_t wsQueueMax = 4,              // max items in wsQueue
                  wsQueueLen = 71;             // max websocket payload len (as "miscellany.h" const uint8_t bufLen)
    uint8_t wsQueueItems = 0;                  // wsQueue buffer counter
    char wsQueue[wsQueueMax][wsQueueLen + 1];  // receive buffer

    /** create a  WebSockets Server **/
    WebSocketsServer webSocket = WebSocketsServer(myPORT);

    /** set time server **/
    //uint16_t NtpPort = 8888,         // local port to listen for UDP packets
    //NtpMs Ntp(ntpServerName, NtpPort, syncOffset);
    NtpMs tms((char *)ntpServerName);  // using the default port and offset values (8888, 1800)
  #endif
  #if defined(SET_AS_NODE_WS) || defined(SET_AS_NODE_00)
    // =============================  https://github.com/esp8266/Arduino
    #include <SoftwareSerial.h>       // you can use direct wired Serial communication between the WEB module and this Master Node

    /** we use separate serial buffers **/
    char swBufOut[bufLen + 1],        // serial out buffer
         swBufIn[bufLen + 1];         // serial in buffer
    boolean swMessage =  false;       // if true: a serial message received

    /** RX, TX, inverse, buffer size **/
    SoftwareSerial swSer(rxPin, txPin, false, 256);  // wired with Node "00" or use a set of HC-05 Bluetooth<-->Serial Master/Slave modules
  #endif
  /** pin conversion char* to int, use function pinToD(char * p) **/
  const uint8_t pinD[] = {D0, D1, D2, D3, D4, D5, D6, D7, D8},
                pinA[] = {A0};   // , A1, A2, A3, A4, A5, A6, A7};  // ESP8266: only one A-pin
  typedef struct FIELDS {
    char to[nToLen + 1];             // to Node<n>  (for my or ... or 99(for ALL) or WS(for the WEBSOCKET module))
    char from[nFromLen + 1];         // from Node<n> (from ... or WS(WEBSOCKET module))
    char function[nfuncLen + 1];     // the Node can have more than one function (relays, LED, ...)
    char command[nCmdLen + 1];       // command in conjunction with function 
    char data[4][nDataLen + 1];      // four data fields, use them as needed
    char timestamp[nTstampLen + 1];  // timestamp in millis
    char mode[nModeLen + 1];         // 0: command/request, 1: response (on request from my)
  }; FIELDS inBuf, outBuf;
  enum {
    NO,   // no action
    ON,   // or DOWN
    OFF,  // or UP
    SAVE  // only save
  };
  simpleTimers askTimer(10000, true);  // ask Node ?? interval starts with n sec.  const uint8_t nToLen = 2,          // (Node)ID  "00"-"nn" or: "WS" Server, "WB" Web Browser
  #ifdef SET_AS_NODE_WS
    void setupWS();
    void loopWS();
   /** webSocket Event **/
    void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t len);
    /** softwareserial input from Node0 module **/
    void swReceived();
    /** serial input for my, browser/App or node(s)? **/
    void doReceiveSw();
  #endif
  #ifdef SET_AS_NODE_00
    /** serial input for my, a other or all? **/
    void swReceived();
    /** softwareserial input from web module or browser/App **/
    void doReceiveSw();
    void setupSW();
    void loopSW();
  #endif
  //=====================
  void setupNode();
  void loopNode();
  /** node receive callback **/
  void receivedCallback(uint32_t from, String &msg);
  /** put data in outBuf structure **/
  void setOutBuf(char *_to, char *_from, char *_function, char *_command, char *data0_, char *_data1, char *_data2, char *_data3, char *_result, char *_timestamp, char *_mode);
  /** copy inBuf to outBuf, change 'to' with 'from', insert data-0-1-2 and set mode to "1" **/
  void setInBufToOutBuf(char *data0);
  void setInBufToOutBuf(char *data0, char *data1);
  void setInBufToOutBuf(char *data0, char *data1, char *data2);
  void setInBufToOutBuf(char *data0, char *data1, char *data2, char *data3);
  void setInBufToOutBuf(String data0, String data1, String data2, String data3);
  /** put (split char string) data string in structure inBuf **/
  void setInBuf(char *str); 
  /** put setOutBuf structure in a string **/
  String setOutStr();
  /** put data in a string **/
  String setOutStr(char *_to, char *_from, char *_function, char *_command, char *_data0, char *_data1, char *_data2, char *_data3, char *_timestamp, char *_mode);
  /** ask Temp and Lux from Node nn **/
  void ascTmpLux(const char *fromNode);  
  /** used by time library to sync the time **/
  time_t syncProvider();
  /** ask for Time from .... **/ 
  void askTime();
  /** get the millis after _start with overflow correction **/
  uint32_t getMillis(uint32_t _start);  
  /** get the pin, repetitions and duration **/
  void setPin(boolean action);
  /** pin conversion from char to D-pin **/
  uint8_t pinToD(char * p);
  /** pin conversion from char to A-pin **/
  uint8_t pinToA(char * p);
  #ifdef USE_PUBLISH
    uint8_t flashPin = greenPin,
            buttonState;
    uint32_t msIntr = millis();
    boolean ledON = false;
    simpleTimers flash(1, true);  // Led flash timer
    void setupPublish();
    void loopPublish();
    void setPublishLED();
    void sendPublish();
  #endif
#endif
