/** simpleHomeSwitch    **
 *  www.kendziorra.nl    *
 *  Ewald sr, 2018       *
 *  totally free to use  *
 *************************
 * basic code
 *************************/

/** FIRST TO DO:
 *    The RTC/TIME must set to UTC/GMT time!
 *    Edit "EditMe.h"          ============
 *  =====================================================
 *  ALL Nodes:
 *    Test incoming data in "nodeReceived()" (see below).
 *    Put your own Functions and commands in "doFunc.ino".
 *  =====================================================
 *  Minimum configuration: Node "WS" and node "00"
 **/

#include "miscellany.h"

//=======================================================//
//##### Three code blocks: "WS", "00" and "01"-"98" #####//
//   Use the "EditMe.h" directives to activate a block   //
//=======================================================//
//                 Node "WS" code                        //
//=======================================================//
#ifdef SET_AS_NODE_WS  // if set as Node "WS" then 
                       //   FIRST TO DO: Edit nodeID "WS"(WebSocket) data in "EditMe.h"
                       //   ===========================================================
                       //   Optional: you can add WiFi OTA to the "WS" code
                       //             On all the other Nodes you can use Serial-Bluetooth OTA
  /** Node/browser/App message processing **/
  void nodeReceived(boolean meshBuf) {
    #ifdef USE_DEBUG
    if (meshBuf) printf("nInBuf: %s\n", nInBuf);
    else printf("swBufIn: %s\n", swBufIn);
    #endif
      if (meshBuf) strcpy(bufIn, (char *)nInBuf);
      else strcpy(bufIn, (char *)swBufIn);
    #ifdef USE_DEBUG
    printf("NodeWS nodeReceived: %s\n", bufIn);
    #endif
    /** data is excusive for my or all? **/
    if (strncmp(bufIn + 1, nodeID, 2) == 0 || strncmp(bufIn + 1, "99", 2) == 0) {
      /** parse data in inBuf structure **/
      setInBuf(bufIn);
      /** is it a command/request bufIn.mode 0 or response bufIn.mode 1 ? **/
      /**                         ============             ============   **/
      if (strcmp(inBuf.mode, "1") == 0) {  // is it a response?
        // response code here
      }
      else {  // it is a command/request
        /** use the inBuf.x structure to message processing **/
        #ifdef USE_DEBUG
        printf("function: %s, command: %s, mode: %s \n", inBuf.function, inBuf.command, inBuf.mode);
        #endif
        switch (atoi(inBuf.function)) {
          case 0: function0(); break;  // unit ON/OFF actions
          case 1: function1(); break;  // unit setup
          case 2: function2(); break;  // unit mode and mode parameters
          case 3: function3(); break;  // Other, not unit, commands and function
        }
      }
    }
    /** from WEB: for all(99) or not for me? **/
    if (strncmp(bufIn + 4, "99", 2) == 0 || strncmp(bufIn + 1, nodeID, 2) != 0 ) sendMessage(String(bufIn));
  }
  
  /** send message to Node(s) or browser/App **/
  void sendMessage(String str) {
    if (str.startsWith("|WB|")) webSocket.broadcastTXT(str);  // to browser/App
    else {  // to the node(s)
      str.setCharAt(0, '@');
      str.setCharAt(str.length() -1, '#');
      swSer.print(str);
      swSer.flush();
      delay(10);
    }
    #ifdef USE_DEBUG
    printf("Node%s sendMessage: %s\n", nodeID, str.c_str());  // demo code
    #endif
  }
  
//=======================================================//
//                 Node "00" code                        //
//=======================================================//
#elif defined(SET_AS_NODE_00)  // Optional: On this Node you can use Serial-Bluetooth OTA
  /** Node message processing **/
  void nodeReceived() {
    nodeReceived(true);
  }
  void nodeReceived(boolean meshBuf) {
    if (meshBuf) {
      strcpy(bufIn, (char *)nInBuf);
      /** to ALL (99) but not send from WEB or NodeWS then sent it also to NodeWS **/
      String str = String(bufIn);
      if (str.startsWith("|99|") && !str.startsWith("|99|W")) {
        str.replace("|99|", "|WS|");
        sendMessage(str);  // sent it also to WS module
        str.replace("|WS|", "|WB|");
        delay(20);
        sendMessage(str);  // sent it also to WeB module
      }
    }
    else strcpy(bufIn, (char *)swBufIn);
  #ifdef USE_DEBUG 
  printf("Node%s nodeReceived: %s\n", nodeID, bufIn);  
  #endif
    /** data is excusive for my or all? **/
    if (strncmp(bufIn + 1, nodeID, 2) == 0 || strncmp(bufIn + 1, "99", 2) == 0) {
    /** parse data in inBuf structure **/
      setInBuf(bufIn);
      /** use the inBuf.x structure to message processing **/
  #ifdef USE_DEBUG 
  printf("function: %s, command: %s, mode: %s \n", inBuf.function, inBuf.command, inBuf.mode);
  #endif
      switch (atoi(inBuf.function)) {
        case 0: function0(); break;  // unit ON/OFF actions
        case 1: function1(); break;  // unit setup 
        case 2: function2(); break;  // unit mode and mode parameters
        case 3: function3(); break;  // Other, not unit, commands and function
      } 
    }
    else sendMessage((char *)nInBuf);  // sent it also to ...
  } 
  
  /** sent message to Node(s) or browser/App **/
  void sendMessage(String str) {
    if (str.startsWith("|W")) {
      str.setCharAt(0, '@');
      str.setCharAt(str.length() -1, '#');
      swSer.print(str);
      swSer.flush();
      delay(10);
    }
    else mesh.sendBroadcast(str);
  #ifdef USE_DEBUG 
  printf("Node%s sendMessage: %s\n", nodeID, str.c_str());  
  #endif
  }
  
//=======================================================//
//                 Node "01"-"98" code                   //
//=======================================================//
#else  // Optional: On these Nodes you can use Serial-Bluetooth OTA
  /** Node message processing **/
  void nodeReceived() {
    strcpy(bufIn, (char *)nInBuf);
    #ifdef USE_DEBUG
      printf("Node%s nodeReceived: %s\n", nodeID, bufIn);
    #endif
    /** data is excusive for my or all? **/
    if (strncmp(bufIn + 1, nodeID, 2) == 0 || strncmp(bufIn + 1, "99", 2) == 0) {
      /** parse data in inBuf structure **/
      setInBuf(bufIn);
      /** use the inBuf.x structure to message processing **/
      #ifdef USE_DEBUG
        printf("function: %s, command: %s, mode: %s \n", inBuf.function, inBuf.command, inBuf.mode);
      #endif
      switch (atoi(inBuf.function)) {
        case 0: function0(); break;  // unit ON/OFF actions
        case 1: function1(); break;  // unit setup
        case 2: function2(); break;  // unit mode and mode parameters
        case 3: function3(); break;  // Other, not unit, commands and function
      }
    }
    // else do notting
  }

  /** sent message to Node(s) or browser/App **/
  void sendMessage(String str) {
    #ifdef USE_DEBUG
      printf("Node%s sendMessage: %s\n", nodeID, str.c_str());  
    #endif
    mesh.sendBroadcast(str);
  }
#endif
//=======================================================//
//             END OFF Node "01"-"98" code               //
//=======================================================//

void setup() {
  #ifdef SET_AS_NODE_WS
    setupWS();
  #elif defined(SET_AS_NODE_00)
    setup00();
  #endif
  setupNode();
  #ifdef USE_PUBLISH
    setupPublish();
  #endif
}

void loop() {
  #ifdef SET_AS_NODE_WS
    loopWS();
  #elif defined(SET_AS_NODE_00)
    loop00();
  #endif
  loopNode();
  #ifdef USE_PUBLISH
    loopPublish();
  #endif
  yield();
}

