﻿#include "simpledb.h"

void sHS_DB::dbInit() {
  db.initDB(nameDB, backupDB, keyLen, recLen, maxRec);
}

/** save a record from record[] **/
void sHS_DB::writeRec(char * key) {     
  char str[db.dtW + 1];
  /** get a simpleDB 'table' string and save it **/
  strcpy(str, "\0");
  for (uint8_t i = 0; i < enumLen; i++) sprintf(str + strlen(str), "%s|", record[i]);      
  db.writeRec(key, str);  // add or, if key exist, overwrite record 
}

/** get and parse a "unit" record to record[] **/
void sHS_DB::readRec(char * key) {
  db.readRec(key);  // record is in db.dbB
  if (db.dbResult != REC_EXIST) return;
  parseRec();
}

void sHS_DB::getRec(uint8_t nr) {
  db.getRec(nr);  // record is in db.dbB
  if (db.dbResult) return;
  /** extract key **/
  strncpy(recKey, db.dbB, 4);
  recKey[4] = '\0';
  /** get record **/
  strcpy(db.dbB, db.dbB + keyLen + 2);
  parseRec();
}
  
/** parse a "unit" record to record[] **/
void sHS_DB::parseRec() {
  char *v, *b;
  uint8_t w = db.dtW;
  char s[strlen(db.dbB)];
  strcpy(s, db.dbB);
  for (uint8_t i = 0; i < enumLen; i++) strcpy(record[i], " ");  // clear values
  while (s[--w] == ' ') s[w] = '\0';  // remove trailing white-space (added by addRec())
  uint8_t i = 0;
  v = strtok_r(s, "|", &b);
  strcpy(record[i++], v); 
  while (v = strtok_r(NULL, "|", &b)) strcpy(record[i++], v);
}

/** init the default values of a new unit record **/
void sHS_DB::newRecord() {
  char *v, *b;
  char r[] = "0|0|6|30|11|30|360|60|21.5|21.0| | |0| | | |0| | |2|0|0|15|15|10|10|D3|D3|-1|-1|-1|";
  uint8_t i = 0;
  v = strtok_r(r, "|", &b);
  strcpy(record[i++], v);
  while (v = strtok_r(NULL, "|", &b)) strcpy(record[i++], v);
  /** result in: record[] filled with default data
      (added to record key and thombstone "kkkk|t|" fields)
    strcpy(record[mode],             "0");   // (position 7 in record)
    strcpy(record[pause],            "0");   // (         9 .... and so on with flexible record fields width)
    strcpy(record[hh1],              "6");   // hh time ON/DOWN
    strcpy(record[mm1],             "30");   // mm
    strcpy(record[hh2],             "11");   // hh time OFF/UP
    strcpy(record[mm2],             "30");   // mm
    strcpy(record[lux1],           "360");   // ON/DOWN luminance
    strcpy(record[lux2],            "60");   // OFF/UP  luminance
    strcpy(record[tmp1],          "21.5");   // ON/DOWN temperature in °C (°F? change "boolean fahrenheit" in sHS_server.h to "true")
    strcpy(record[tmp2],           "21.0");  // OFF/UP  temperature in °C
    strcpy(record[code],             " ");   // KaKu learn (Old (code) and New (address))
    strcpy(record[period],           " ");   // .
    strcpy(record[status],           "0");   // .      status (unit is: ON[DIM]/DOWN or OFF/UP)
    strcpy(record[address],          " ");   // .
    strcpy(record[groupBit],         " ");   // .
    strcpy(record[unit],             " ");   // .
    strcpy(record[dimLevelPresent],  "0");   // .
    strcpy(record[dimLevel],         " ");   // (last KaKu)
    strcpy(record[pin],              " ");   // last active pin
    strcpy(record[repetitions],      "2");   // repetition pin HICH/LOW
    strcpy(record[duration],         "0");   // duration in ms between repetition
    strcpy(record[reversed],          "0");   // ON/DOWD<=>OFF/UP
    strcpy(record[riseOffset],      "15");   // sunRise offset minutes
    strcpy(record[setOffset],       "15");   // sunSet offset minutes
    strcpy(record[yoyo],            "10");   // anti-YoYo (Lux and Temp) in minutes
    strcpy(record[badLux],          "10");   // bad weather
    strcpy(record[pin1],            "D3");   // pin ON/DOWN (if one optocoupler/relay/... also OFF/UP)
    strcpy(record[pin2],            "D3");   // pin OFF/UP
    strcpy(record[counter1],        "-1");   // used by timer or Lux/Temp anti-Yoyo (else free to use)
    strcpy(record[counter2],        "-1");   // used by timer or Lux/Temp anti-Yoyo (else free to use)
    strcpy(record[counter3],        "-1");   // used by Lux/Temp anti-Yoyo (else free to use)
  **/
}

