/** 28YBJ-48 DC 5V 4 Phase 5 Wire Stepper Motor With ULN2003 Driver Board - 4096 steps **/

#include "simpleStepperSteps.h"

StepperSteps::StepperSteps(
             uint8_t _stepPin1,
             uint8_t _stepPin2, 
             uint8_t _stepPin3, 
             uint8_t _stepPin4, 
             uint8_t _homePin, 
             int16_t _endPin,  
             boolean _rotSteps,
             uint8_t _stepSpeed,
             uint16_t _maxRotations) {
  in1234[0] = _stepPin1;
  in1234[1] = _stepPin2;
  in1234[2] = _stepPin3;
  in1234[3] = _stepPin4;
  homePin = _homePin;
  endPin = _endPin;
  rotSteps = _rotSteps;
  stepSpeed = _stepSpeed;
  maxRotations = _maxRotations;
  
  for (uint8_t i = 0; i <= 3; i++) pinMode(in1234[i], OUTPUT);
  pinMode(homePin, INPUT_PULLUP);
  gotoHome();  // FIRST: find the HOME position 
  if (endPin >= 0) {
    pinMode(endPin, INPUT_PULLUP);
    gotoEnd();   // FIRST: find the END position 
    endPos = stepPos;
    gotoHome();  // start point is position 0
  }
  else endPos = abs(_endPin);  
}

/** find the home position **/
void StepperSteps::gotoHome() {  // turn LEFT
  findHomeEnd(true);
  stepPos = 0;
}

/** find the end position **/
void StepperSteps::gotoEnd() {  // turn RIGHT
  if (endPin >= 0) findHomeEnd(false);
  else stepTo(endPos);
}

void StepperSteps::findHomeEnd(boolean toHome) {
  for (uint16_t r = 0; r < maxRotations; r++) {  // The maximun number of rotations to find the home/end position
    for (uint16_t i = 0; i <= 511; i++) {  // 512 steps is one 360* rotation
      if ((toHome ? isHome : isEnd)) return;
      stepper((toHome ? -1 : 1));  // turn left/right
    }
  }
}

/** turn to the requested position in <n> steps  **/
void StepperSteps::stepTo(uint16_t nSteps) {
  int16_t n = -(stepPos - nSteps);
  stepper(n);
}

/** do one stepper step **/
void StepperSteps::setSteps(const uint8_t out[4]) {
  for (uint8_t i = 0; i <= 3; i++) digitalWrite(in1234[i], out[i]);
  if (digitalRead(homePin) == LOW) {isHome = true; stepPos = 0;}
  else if (endPin >= 0 && digitalRead(endPin) == LOW) isEnd = true;
  else if (stepPos >= endPos) isEnd = true;
}

/** do <n> steps  **/
void StepperSteps::stepper(int16_t nSteps) {
  if (nSteps == 0) return;
  if (rotSteps) {smallSteps(nSteps); return;}  // 0-4095 steps  = 360*
  // (0-511 steps = 360*)
  uint16_t stps = abs(nSteps);
  isHome = isEnd = false;
  if (nSteps > 0) {
    for (uint16_t n = 0; n < stps; n++)  // turn + (R)
      stepPos++;
      for (uint8_t i = 0; i <= 7; i++) {
        setSteps(steps[i]);
        if (isEnd) break;
        delay(stepSpeed);}
  }
  else {
    for (uint16_t n = 0; n < stps; n++)  // turn - (L)
      stepPos--;
      for (int8_t i = 7; i >= 0; i--) {
        setSteps(steps[i]);
        if (isHome) break;
        delay(stepSpeed);}
  }
  setSteps(steps[8]);  // stepper default values
}

/** turn to the requested position in <n> steps **/// (0-4095 steps = 360*)
void StepperSteps::smallSteps(int16_t nSteps) {  // one nSteps = 1 stepper step, one rotation = 4096 nSteps
  uint16_t stps, s;
  isHome = isEnd = false;
  stps = s = abs(nSteps);
  if (nSteps > 0) {
    for (uint16_t n = 0; n < stps; n++) {  // turn + (R)
      for (uint8_t i = 0; i <= 7; i++) {
        stepPos++;
        setSteps(steps[i]); 
        if (isEnd || (--s == 0)) break;
        delay(stepSpeed);
      }
      if (isEnd || (s == 0)) break;
    }
  }
  else {
    for (uint16_t n = 0; n < stps; n++) {  // turn - (L)
      for (int8_t i = 7; i >= 0; i--) {
        stepPos--;
        setSteps(steps[i]); 
        if (isHome ||  (--s == 0)) break;
        delay(stepSpeed);
      }
      if (isHome || (s == 0)) break;
    }
  }
  setSteps(steps[8]);  // stepper default values
}
