
#ifndef SIMPLESTEPPER_H
#define SIMPLESTEPPER_H

#include <Arduino.h>

class StepperSteps {
  public:
    StepperSteps(uint8_t _stepPin1,
                 uint8_t _stepPin2, 
                 uint8_t _stepPin3, 
                 uint8_t _stepPin4, 
                 uint8_t _homePin, 
                 int16_t _endPin,    // nn>=0: endPin microswitch END, nn<0: end position steps (no END microswitch)
                 boolean _rotSteps,  // stepper mode, false: 512 steps is 360*, true: 4096 steps is 360*
                 uint8_t _stepSpeed,
                 uint16_t _maxRotations
    );
    
    uint16_t stepPos = 0,  // stepper position
             endPos = 0;   // last step position (HOME to END position in 512 or 4096 steps)
    
    void gotoHome();
    void gotoEnd();
    void stepper(int16_t);  // 0- 511 = 360* or 0-4095 = 360*
    void stepTo(uint16_t);  // go to step nn (from HOME), 512 or 4096 steps is 360*
    
  private:
    void smallSteps(int16_t);  // 0-4095 = 360*
    void setSteps(const uint8_t *);
    void findHomeEnd(boolean toHome);
    
    const uint8_t steps[9][4] = {  // 8 steps and one default (steps[8])
                    {0, 0, 0, 1}, {0, 0, 1, 1}, {0, 0, 1, 0}, {0, 1, 1, 0},
                    {0, 1, 0, 0}, {1, 1, 0, 0}, {1, 0, 0, 0}, {1, 0, 0, 1},
      /*default ->*/{0, 0, 0, 0}};/*<- default*/
    uint8_t in1234[4] = {D5, D6, D7, D8},  // stepper pins on WeMos/NodeMCU/...  (ESP8266 GPIO 14/12/13/15)
            homePin = D0,   // microswitch home position, pin with pull-up on WeMos/NodeMCU/...  (ESP8266 GPIO-16)
            endPin = D3,    // microswitch end position, pin with pull-up on WeMos/NodeMCU/...  (ESP8266 GPIO-0)
            stepSpeed = 1;  // stepper speed, delay in ms,  default 1, min: 1, max: 50
    uint16_t maxRotations = 5;  // The maximun number of rotations to find the home/end position
    boolean isHome,
            isEnd,
            rotSteps;  // false: 512 steps is 360*, true: 4096 steps is 360*
};

#endif

