#include <ESP8266WiFi.h>
#include <NtpMs.h>    // kendziorra.nl
#include <simpleSummerTimeDST.h>

char mySSID[] = "edit-edit",
     myPASSWORD[] = "edit-edit-edit-edit",
     ntpServerName[] = "edit-edit-edit";  // "nl.pool.ntp.org" or ....

uint16_t syncOffset = 1800;  // time sync after each n seconds; small value = more precision
//uint16_t localPort = 8888,   // local port to listen for UDP packets
//NTPms Ntp(ntpServerName, localPort, syncOffset);
NTPms Ntp(ntpServerName);  // using the default port and offset values (8888, 1800) 

DST dst( 
    /** DST/Summer Time rules. Change it for a other area **
     ** ================================================= **/
    /** EU summer time rules:
     *    start: last Sunday in March   at 01:00 add      1h   to local time
     *    end:   last Sunday in October at 01:00 subtract 1h   from local time
     **/
  /*startMonth*/    3,
  /*endMonth*/     10,   // 1 - 12 (jan - dec)
  /*startMonthDay*/ 0,
  /*endMonthDay*/   0,   // 1 - 31, not used = 0 (if using ...WeekDay)
  /*startWeekDay*/  1, 
  /*endWeekDay*/    1,   // 1 -  7 (Sun - Sat), not used = 0 (if using ...MonthDay)
  /*startHour*/     1, 
  /*endHour*/       1,   // 0 - 23 (UTC)
  /*startMinute*/   0, 
  /*endMinute*/     0,   // 0 - 59 (Lord Howe Island rules use a half-hour time zone ;=))
  /*startWeek*/    -1, 
  /*endWeek*/      -1,   // 1 -  4, last week of month: -1 (4 or 5)
  /*dstOffset*/  3600,   // dst/summertime offset in seconds
  /*hemisphere*/ true,   // Northern Hemisphere: true,  Southern Hemisphere: false
  /*timeZone*/     1);   // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))

void setup() {
  Serial.begin(115200);
  Serial.setDebugOutput(true); 
  Serial.println('\n');
  WiFi.disconnect();  // always close WiFi (after a firmware flash: a previous open connection always remains in memory)
  WiFi.mode(WIFI_STA);
  WiFi.begin(mySSID, myPASSWORD);       // used by NHS Browser clients and other NHS IoThing Slave(s)
  while (WiFi.waitForConnectResult() != WL_CONNECTED) delay(500);
  /** wait for NTP-time Server **/
  while (!Ntp.timeIsSynced) {
    Ntp.syncTheTime();
    delay(200);
  }

  /** DST calculation **/
  int8_t saveDay = -1;  // normal a global var.
  time_t t = now();
  printf("\n\nUTC/GMT time: \t\t%02u-%02u-%04u %02u:%02u:%02u\n", month(t), day(t), year(t), hour(t), minute(t), second(t));

  if (day(t) != saveDay) {  // normal used in loop()
    dst.calcDst(t);  // first use this on a every new day!!!
    saveDay = day(t);
  }
  boolean isDST = dst.isItDst(t, false);   // return true/false - if true: use calcDst([time]) first
  printf("DST?: \t\t\t%s\n", (isDST ? "YES" : "NO"));

  time_t l = dst.localTime(t, false);  // return local time - true: if use calcDst([time]) first
  printf("Local time: \t\t%02u-%02u-%04u %02u:%02u:%02u\n", month(l), day(l), year(l), hour(l), minute(l), second(l));
  
  time_t s = dst.dstStart;          // is set after calcDst([time])
  printf("dstStart UTC time: \t%02u-%02u-%04u %02u:%02u:%02u\n", month(s), day(s), year(s), hour(s), minute(s), second(s));
  
  time_t e = dst.dstEnd;              // is set after calcDst([time])
  printf("dstEnd UTC time: \t%02u-%02u-%04u %02u:%02u:%02u\n\n", month(e), day(e), year(e), hour(e), minute(e), second(e));

}

void loop() {
//  Ntp.updateTime();

}
