/***********************************************
 * DST/SUMMERTIME by Ewald Kendziorra sr, 2017 *
 ***********************************************/
// Code: from here and there ;=)

#ifndef SIMPLEDST_H
#define SIMPLEDST_H

#include <Arduino.h>
#include <TimeLib.h>  // https://github.com/PaulStoffregen/Time

class DST {
  public:
    DST(uint8_t _startMonth,     // 1 - 12 (jan - dec) 
        uint8_t _endMonth,       //   ...
        uint8_t _startMonthDay,  // 1 - 31, not used = 0 (if using ...WeekDay) 
        uint8_t _endMonthDay,    //   ...
        uint8_t _startWeekDay,   // 1 -  7 (Sun - Sat), not used = 0 (if using ...MonthDay)
        uint8_t _endWeekDay,     //   ...
        uint8_t _startHour,      // 0 - 23 (UTC)
        uint8_t _endHour,        //   ...
        uint8_t _startMinute,    // 0 - 59 (Lord Howe Island rules use a half-hour time zone ;=))
        uint8_t _endMinute,      //   ...
        int8_t  _startWeek,      // 1 -  4, last week of month: -1 (4 or 5)
        uint8_t _endWeek,        //   ... 
        uint16_t _dstOffset,     // dst/summertime offset in seconds
        boolean _hemisphere,     // Northern Hemisphere: true,  Southern Hemisphere: false
        double _timeZone         // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))
    );

    const uint16_t oneHour = 3600;
    const uint32_t oneDay  = 86400, oneWeek = 604800;
    
    void calcDst();                     // DST calc, using time now()   (UTC)
    void calcDst(time_t);               // DST calc, using a other time (UTC)
    boolean isItDst(time_t, boolean);   // return true/false - if true: use calcDst([time]) first
    time_t localTime(time_t, boolean);  // return local time - if true: use calcDst([time]) first
    
    time_t dstStart, dstEnd;            // is set after calcDst(), calcDst(time_t)
    
    /** extra date/time functions **/
    time_t calcTimeStamp(uint8_t, uint8_t, uint8_t, uint8_t, uint8_t, uint16_t);
    uint16_t yearDay(time_t);
    uint16_t yearDay(uint8_t, uint8_t, uint16_t);
    boolean leapYear(uint16_t);
    uint16_t dstOffset = 3600;  // dst/summertime offset in seconds
    double  timeZone = 1;       // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))

  private:        
    /** EU summer time rules:
     *    start: last Sunday in March   at 01:00 add      1h   to local time
     *    end:   last Sunday in October at 01:00 subtract 1h   from local time
     **/
    
    uint8_t startMonth    =  3, endMonth    = 10,  // 1 - 12 (jan - dec)
            startMonthDay =  0, endMonthDay =  0,  // 1 - 31, not used = 0 (if using ...WeekDay)
            startWeekDay  =  1, endWeekDay  =  1,  // 1 -  7 (Sun - Sat), not used = 0 (if using ...MonthDay)
            startHour     =  1, endHour     =  1,  // 0 - 23 (UTC)
            startMinute   =  0, endMinute   =  0;  // 0 - 59 (Lord Howe Island rules use a half-hour time zone ;=))
    int8_t  startWeek     = -1, endWeek     = -1;  // 1 -  4, last week of month: -1 (4 or 5)
    boolean hemisphere = true;  // Northern Hemisphere: true,  Southern Hemisphere: false
};

#endif

