#include <ESP8266WiFi.h>
#include <NtpMs.h>    // kendziorra.nl
#include <simpleSummerTimeDST.h>
#include <simpleSunRiseSet.h>

char mySSID[] = "edit-edit";
char myPASSWORD[] = "edit-edit-edit-edit";
char ntpServerName[] = "edit-edit-edit";  // "nl.pool.ntp.org" or ....

uint16_t syncOffset = 1800;  // time sync after each n seconds; small value = more precision
//uint16_t localPort = 8888,   // local port to listen for UDP packets
//NTPms Ntp(ntpServerName, localPort, syncOffset);
NTPms Ntp(ntpServerName);  // using the default port and offset values (8888, 1800) 

time_t getTime() { 
  if (!Ntp.timeIsSynced) Ntp.syncTheTime();  // NTP time OK?
  return Ntp.UNIXtimestamp;  // UTC
}

double timeZone = 1;                  // to set/get the UTC/GMT or local time (Lord Howe Island rules use a half-hour time zone ;=))
int16_t dstOffset = 3600;             // in seconds to set/get the UTC/GMT or local time
DST dst( 
    /** DST/Summer Time rules. Change it for a other area **
     ** ================================================= **/
    /** EU summer time rules:
     *    start: last Sunday in March   at 01:00 add      1h   to local time
     *    end:   last Sunday in October at 01:00 subtract 1h   from local time
     **/
  /*startMonth*/    3,
  /*endMonth*/     10,   // 1 - 12 (jan - dec)
  /*startMonthDay*/ 0,
  /*endMonthDay*/   0,   // 1 - 31, not used = 0 (if using ...WeekDay)
  /*startWeekDay*/  1, 
  /*endWeekDay*/    1,   // 1 -  7 (Sun - Sat), not used = 0 (if using ...MonthDay)
  /*startHour*/     1, 
  /*endHour*/       1,   // 0 - 23 (UTC)
  /*startMinute*/   0, 
  /*endMinute*/     0,   // 0 - 59 (Lord Howe Island rules use a half-hour time zone ;=))
  /*startWeek*/    -1, 
  /*endWeek*/      -1,   // 1 -  4, last week of month: -1 (4 or 5)
  /*dstOffset*/  dstOffset,   // dst/summertime offset in seconds
  /*hemisphere*/ true,   // Northern Hemisphere: true,  Southern Hemisphere: false
  /*timeZone*/   timeZone);   // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))

sunRiseSet riseSet(dstOffset,  // dst/summertime offset in seconds
                   52,         // The latitude is preceded by a minus sign ( – ) if it is south of the equator (a positive number implies north)
                   5,          // the longitude is preceded by a minus sign if it is west of the prime meridian (a positive number implies east)
                               //    52, 5: the Centre of the Netherlands
                   timeZone,   // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))
                   90.883);    // zenith:
                               //    offical      = 90.883 degrees (90° 50')
                               //    civil        = 96 degrees
                               //    nautical     = 102 degrees
                               //    astronomical = 108 degrees

void setup() {
  Serial.begin(115200);
  Serial.setDebugOutput(true); 
  Serial.println('\n');
  WiFi.disconnect();  // always close WiFi (after a firmware flash: a previous open connection always remains in memory)
  WiFi.mode(WIFI_STA);
  WiFi.begin(mySSID, myPASSWORD);       // used by NHS Browser clients and other NHS IoThing Slave(s)
  while (WiFi.waitForConnectResult() != WL_CONNECTED) {
    delay(5000);
    ESP.restart();
  }
  /** wait for NTP-time Server **/
  while (!Ntp.timeIsSynced) {
    Ntp.syncTheTime();
    delay(200);
  }
  /** get the time **/
  setSyncProvider(getTime);
  setSyncInterval(syncOffset);

  time_t t = now();
  /** Sun rise/set and Summertime/DST calculation **/
  time_t l = dst.localTime(t, true);  // return local time - true: if use calcDst([time]) first
  printf("\n\nLocal time:\t%02u-%02u-%04u %02u:%02u:%02u\n", month(l), day(l), year(l), hour(l), minute(l), second(l));
  riseSet.calcSun(t, dst.isItDst(t, false));  // if true: calc DST first
  printf("Sun rise:\t%02u:%02u\n", riseSet.sunRiseHour, riseSet.sunRiseMinute);
  printf("Sun set:\t%02u:%02u\n\n", riseSet.sunSetHour, riseSet.sunSetMinute);
}

void loop() {
//  Ntp.updateTime();

}
