#ifndef SIMPLESUNRISESET_H
#define SIMPLESUNRISESET_h

#include <Arduino.h>
#include <TimeLib.h>  // https://github.com/PaulStoffregen/Time

class sunRiseSet {
  public:
    sunRiseSet(uint16_t _dstOffset,  // dst/summertime offset in seconds
               float _lat,           // The latitude is preceded by a minus sign ( – ) if it is south of the equator (a positive number implies north)
               float _lon,           // the longitude is preceded by a minus sign if it is west of the prime meridian (a positive number implies east)
               double _timeZone,     // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))
               float _zenith);       //    offical      = 90.883 degrees (90° 50')
                                     //    civil        = 96 degrees
                                     //    nautical     = 102 degrees
                                     //    astronomical = 108 degrees
    
    void calcSun(boolean);          // calc rise and set times, using time now()   (UTC) - true it's summertime/DST
    void calcSun(time_t, boolean);  // calc rise and set times, using a other time (UTC) - true it's summertime/DST
    uint8_t sunRiseHour, sunRiseMinute, sunSetHour, sunSetMinute;  // use this after calcSun() or calcSun(time_t)

  private:  
    float sunTime(uint16_t, boolean);  // calc decimal time 

    uint16_t dstOffset = 3600;    // dst/summertime offset in seconds
    float lat = 52.0, lon = 5.0;  // Northern hemisphere: centre of the Netherlands
    double timeZone =  1;         // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))
    float zenith = 90.883;
    // zenith
    //    offical      = 90.883 degrees (90° 50')
    //    civil        = 96 degrees
    //    nautical     = 102 degrees
    //    astronomical = 108 degrees
    const float toRad = 0.01745329, toDeg = 57.2957786;
};

#endif

