#include <simpleThermistor.h>

/** Steinhart-Hart model equation
 *    to get the right A,B and C values
 * Input resistance-temperature pairs
 *   R (Ω)       T (°C) Rn-values: Thermistor resistans by Tn (use a mulimeter)
 *   ------------------ Tn-values: based on the required temperature range (use a thermometer)
 *   R1: 25000   T1:  5 (EXAMPLE VALUES)
 *   R2: 10000   T2: 25
 *   R3:  4000   T3: 45
 *===========================================================   
 *   R = 10000 Ω  series resistor resistans ~25*C (use a mulimeter) (EXAMPLE VALUE)
 *       value based on the 25*C Thermistor resistans (use a mulimeter)
 * schematic
 * (GND) ---- (Resistor) ----|---- (Thermistor) ---- (+VCC)
 *                           |
 *                       Analog PINn
 **/ 
 
// (EXAMPLE VALUES Thermistor resistans by temperature range 0-50*C)
// use a mulimeter and thermometer to get the right values
// Tn-values: based on the required temperature range
double R1 = 25000.0, T1 =  5.0,  // Low resistans on temperature
       R2 = 10000.0, T2 = 25.0,  // Medium resistans on temperature
       R3 = 4000.0,  T3 = 45.0,  // High resistans on temperature
       R = 10000.0,              // series resistor on room temperature (use a mulimeter)
       S = 1023.0;               // ADC resolution, 1023 Arduino/ESP8266,
                                 //   if using ADS1115: use the value from simpleADS115 library adc.maxADS      

// init Thermistor and calc Steinhart-Hart model equation to get the A, B and C coefficients
Thermistor therm(R1, T1,
                 R2, T2,
                 R3, T3,
                 R,
                 S);

void setup() {
  Serial.begin(115200);
//  Serial.setDebugOutput(true); 
  Serial.println('\n');
  
  uint16_t adc = 511;  // demo value (Thermistor temperature +/- 25*C)
                       // get the adc from a A-PIN or ADS1115 port
                       //         ===        =====

  double K = therm.getTempK(adc),
         C = therm.getTempC(adc),
         F = therm.getTempF(adc);

  printf("\ntemp: %s*K, %s*C, %s*F, R: %u Ohm\n\n", String(K).c_str(), String(C).c_str(), String(F).c_str(), therm.ntcR);
}

void loop() {
}
