#include "simpleThermistor.h"

Thermistor::Thermistor(double r1, double t1,
                       double r2, double t2,
                       double r3, double t3,
                       double r,
                       double s) {
  R1 = r1; T1 = t1;
  R2 = r2; T2 = t2;
  R3 = r3; T3 = t3;
  R = r;
  S = s;
  calcABC();
}

double Thermistor::getTempC(uint16_t adc) {
  return getTempK(adc) - 273.15;
}

double Thermistor::getTempF(uint16_t adc) {
  return (getTempK(adc) - 273.15) * 1.8 + 32.0;
}

/** Steinhart-Hart model equation: calc temperature **/ 
double Thermistor::getTempK(uint16_t adc) {  // adc: ADC value from a A-PIN) - temp as Kelvin
  double R1 = R * ((S / adc) - 1.0),         // calc R (S: DAC resolution)
         L = log(R1),
         T = (uint16_t)(1.0 / (A + B * L + C * pow(L, 3)));
  ntcR = (uint32_t)R1;
  return T;
}

void Thermistor::calcABC() {
  // Solve system of equations to determine coefficients for Steinhart-Hart equation.
  // See: http://en.wikipedia.org/wiki/Steinhart-Hart_equation
  double L1 = log(R1), L2 = log(R2), L3 = log(R3),
         Y1 = 1.0 / (T1 + 273.15), Y2 = 1.0 / (T2 + 273.15), Y3 = 1.0 / (T3 + 273.15),
         G2 = ((Y2 - Y1)/(L2 - L1)), G3 = ((Y3 - Y1)/(L3 - L1));
  C = ((G3 - G2)/(L3 - L2))*pow((L1 + L2 + L3), -1.0);
  B = G2 - C*(pow(L1, 2.0) + L1*L2 + pow(L2, 2.0));
  A = Y1 - (B + pow(L1, 2.0)*C)*L1;
}
