#ifndef SIMPLETHERMISTOR_H
#define SIMPLETHERMISTOR_H

#include <Arduino.h>

class Thermistor {
  public:
    //  Input resistance-temperature pairs
    Thermistor(double r1, double t1,  // Low resistans on temperature
               double r2, double t2,  // Medium resistans on temperature
               double r3, double t3,  // High resistans on temperature
               double r,              // R - series resistor (use a mulimeter)
               double s);             // S - resolution, 1023 Arduino/ESP8266, if using ADS1115 use the value from simpleADS115 library adc.maxADS
               
    // adc: ADC value from a A-PIN or ADS1115 port
    double getTempK(uint16_t adc);
    double getTempC(uint16_t adc);
    double getTempF(uint16_t adc);
    
    uint32_t ntcR = 0;  // holt thermistors resistans value
    
  private:
    /** Steinhart-Hart coefficients calculation
          to get the right A,B and C **/
    void calcABC();
    double R1, T1,
           R2, T2,
           R3, T3,
           A, B, C,
           R,  // series resistor (use a mulimeter)
           S;  // resolution       
};
  
#endif


