/** simpleTimeNtpWebRtcMillis **
 *  www.kendziorra.nl          *
 *  Ewald sr, 2018             *
 *  totally free to use        *
 *******************************/

#include "RtcMs.h"

RtcMs::RtcMs() {
  rtcWriteByte(cntrReg, 0x1c);  // register 0Eh
}

uint8_t RtcMs::rtcReadByte(uint8_t regadd) {
  Wire.beginTransmission(rtcAddr);
  Wire.write((byte)regadd);
  Wire.endTransmission();
  Wire.requestFrom(rtcAddr, (uint8_t)1);
  return Wire.read();
}

void RtcMs::rtcWriteByte(uint8_t regadd, uint8_t val) {
  Wire.beginTransmission(rtcAddr); // address DS3231
  Wire.write(regadd);              // select register
  Wire.write(val);                 // write value
  Wire.endTransmission();
}

time_t RtcMs::getTimestamp() {  // UNIX Timestamp
  calcMs();
  Wire.beginTransmission(rtcAddr);
  Wire.write((byte)0x00);  // sec. reg.
  Wire.endTransmission();
  Wire.requestFrom(rtcAddr, (uint8_t)8);  // read 8 bytes - sec. to year values
  uint8_t s = bcd2bin(Wire.read());
  uint8_t m = bcd2bin(Wire.read());
  uint8_t h = bcd2bin((Wire.read() & 0x7f)); //Ignore 24 Hour bit
  Wire.read();  // not uded
  uint8_t D = bcd2bin(Wire.read());
  uint8_t M = bcd2bin(Wire.read());
  uint16_t Y = bcd2bin(Wire.read()) + 2000;
  return calcTimeStamp(h, m, s, D, M, Y);
}

void RtcMs::setTimestamp(time_t t) {  // UNIX Timestamp
  Wire.beginTransmission(rtcAddr);   // address DS3231
  Wire.write((byte)0x00);  //beginning from SEC Register address
  Wire.write((byte)bin2bcd(second(t))); 
  Wire.write((byte)bin2bcd(minute(t)));
  Wire.write((byte)bin2bcd((hour(t)) & 0x3f)); // 24 hour
  Wire.write((byte)(((t / 86400) + 4) % 7));   // extract weekDay: oneDay=86400sec, 1-1-1970=Thursday weekDay 5
  Wire.write((byte)bin2bcd(day(t)));
  Wire.write((byte)bin2bcd(month(t)));
  Wire.write((byte)bin2bcd(year(t) - 2000));  
  Wire.endTransmission();
  setTime(t);
}

/** correct now() <and RTC time> with (+/-nn) seconds **/
void RtcMs::correctTime(int32_t s) {
  setTimestamp(now() + s);
  getTimestamp();
}

void RtcMs::calcMs() {
  uint32_t stop, start, difference;
  stop = millis();
  start = startMillis;
  startMillis = millis();
  if(stop < start)  // overflow=true
    difference = (uint32_t)4294967295 - start + stop + 1;
  else                // overflow=false
    difference = stop - start; 
  difference += milliseconds; 
  milliseconds = difference % 1000;
  startMillis = millis() + 1; // + 0.001 sec. work time compensation
}

char * RtcMs::getMsTimestamp() { // timestamp STRING in millis (UNIX * 1000)
  char msT[14];  // timestamp in millis, 14 digits char string
  char m[4];
  calcMs();
  ultoa(now(), msT, 10);
  ultoa(milliseconds, m, 10);
  strcat(msT, "000");
  strcpy(msT + 13 - strlen(m), m);  // add millis to the UNIXtimestamp
  return msT;
}

time_t RtcMs::calcTimeStamp(uint8_t h, uint8_t m, uint8_t s, uint8_t D, uint8_t M, uint16_t Y) {  // get UNIX time_t Time Stamp from hh:mm:ss DD-MM-YY
  // UNIX timestamp = number of seconds since 1 Jan 00:00:00 1970
  if (Y < 100) Y += 2000;
  time_t t = 0;
  for (uint16_t i = 1970; i < Y; i++) t += yearDay(31, 12, i) * oneDay;
  return t + (yearDay(D, M, Y) - 1) * oneDay + (uint32_t)h * oneHour + (uint32_t)m * 60 + s;
}

uint16_t RtcMs::yearDay(uint8_t d, uint8_t m, uint16_t y) {  // year days from date numbers
  int monthdays[13] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
  return monthdays[m - 1] + (monthdays[m - 1] > 31 && leapYear(y) ? 1 : 0) + d;
}

boolean RtcMs::leapYear(uint16_t y) {  // leap year?
  uint16_t Y = y < 100 ? y + 2000 : y;
  return ((Y % 4 == 0 && Y % 100 != 0) || Y % 400 == 0);
}
