/** simpleTimeNtpWebRtcMillis **
 *  www.kendziorra.nl          *
 *  Ewald sr, 2018             *
 *  totally free to use        *
 *******************************

/** functions
    time_t getTimestamp();        // get UNIX timestamp
    void setTimestamp(time_t t);  // NTP and WEB: DUMMY FUNCTION set time from UNIX timestamp
    void correctTime(int32_t s);  // NTP and WEB: DUMMY FUNCTION time correction in seconds
    char * getMsTimestamp();      // timestamp in millis, 14 digits char string
 **/
 
#include <TimeLib.h>                 // https://github.com/PaulStoffregen/Time

const uint16_t SyncInterval = 1800,  // sync the time interval in seconds
               dstOffset = 3600;     // in seconds to set/get the UTC/GMT or local time
const double timeZone = 1;           // to set/get the UTC/GMT or local time (Lord Howe Island rules use a half-hour time zone ;=))
boolean isDST = false;               // summertime/DST?

#define USE_NTP  // using a NTP Server time
#ifdef USE_NTP
  #include <ESP8266WiFi.h>
  #include <NtpMs.h> 
  const char mySSID[] = "edit-edit"@editThis,
             myPASSWORD[] = "edit-edit-edit-edit"@editThis,
           ntpServerName[] = "time.google.com";  // URL NTP Server, by example: "nl.pool.ntp.org" or "time.google.com" or ....
  NtpMs tms = NtpMs((char *)ntpServerName);
#else  
  //#define USE_WEB  // using Google time
  #ifdef USE_WEB
    #include <ESP8266WiFi.h>
    #include <WebMs.h>
    const char mySSID[] = "edit-edit"@editThis,
               myPASSWORD[] = "edit-edit-edit-edit"@editThis;
    WebMs tms = WebMs();
  #else  
    //#define USE_RTC       // using RTC time
    #ifdef USE_RTC        // first time use? set time with "setTimestamp(time_t t)"
      #define USE_WIRE   // RTC use this
      #include <Wire.h>
      const uint8_t sdaPin = D2,           // pin: SDA
                    sclPin = D1;           // pin: SCL
      #include <RtcMs.h>  //   in "setup()" after "setSyncProvider(syncProvider)"
      RtcMs tms = RtcMs();
    #else  
      //#define USE_TIME       // using "TimeLib.h" time
      #ifdef USE_TIME        // Use function "setTimestamp(time_t t)"
        #include <TimeMs.h>  //   in "setup()" after "setSyncProvider(syncProvider)"
        TimeMs tms = TimeMs();
      #endif
    #endif
  #endif
#endif

time_t syncProvider() {
  return tms.getTimestamp();
}

time_t localTime(time_t t) {
  return t + (uint32_t)(timeZone * 3600 + (isDST ? dstOffset : 0)); 
}

void setup() {
  Serial.begin(115200);
//  Serial.setDebugOutput(true); 
  Serial.println("\n...\n");

  #if defined(USE_NTP) || defined(USE_WEB)
    WiFi.disconnect();  // always close WiFi (after a firmware flash: a previous open connection always remains in memory)
    WiFi.mode(WIFI_STA);
    WiFi.begin(mySSID, myPASSWORD);
    while (WiFi.status() != WL_CONNECTED) delay(500);
  #endif

  #ifdef USE_WIRE  // RTC use this
    Wire.begin(sdaPin, sclPin);
  #endif

  setSyncProvider(syncProvider);  // set the external time provider
  setSyncInterval(SyncInterval);  // set the number of seconds between re-sync

  /** set time **/ 
  #ifdef USE_TIME
    tms.setTimestamp(1510490400);  // insert a UNIX timestamp http://www.unixtimestamp.com/index.php
  #endif
}
 
void loop() {
  char msTs[14];  // timestamp String with millis
  strcpy(msTs, tms.getMsTimestamp());  // usable as websocket timestamp or ...
  time_t u = now();
  time_t t = localTime(u);
  printf("Loc time   : %02u-%02u-%04u %02u:%02u:%02u \n", day(t), month(t), year(t), hour(t), minute(t), second(t));
  printf("UTC time   : %02u-%02u-%04u %02u:%02u:%02u \n", day(u), month(u), year(u), hour(u), minute(u), second(u));
  printf("UTC Millis$: %s \n\n", msTs);
  delay(10000);
}