/** simpleTimeNtpWebRtcMillis **
 *  www.kendziorra.nl          *
 *  Ewald sr, 2018             *
 *  totally free to use        *
 *******************************

/** functions
    time_t getTimestamp();        // get UNIX timestamp
    void setTimestamp(time_t t);  // DUMMY FUNCTION set time from UNIX timestamp
    void correctTime(int32_t s);  // DUMMY FUNCTION time correction in seconds
    char * getMsTimestamp();      // timestamp in millis, 14 digits char string
 **/

#include <ESP8266WiFi.h>
#include <TimeLib.h>               // https://github.com/PaulStoffregen/Time
#include <WebMs.h>

const char mySSID[] = "edit-edit"@editThis,
           myPASSWORD[] = "edit-edit-edit-edit"@editThis;
const uint16_t SyncInterval = 1800,  // sync the time interval in seconds
               dstOffset = 3600;     // in seconds to set/get the UTC/GMT or local time
const double timeZone = 1;           // to set/get the UTC/GMT or local time (Lord Howe Island rules use a half-hour time zone ;=))
boolean isDST = false;               // summertime/DST?

WebMs tms = Web();

time_t syncProvider() {
  return tms.getTimestamp();
}

time_t localTime(time_t t) {
  return t + (uint32_t)(timeZone * 3600 + (isDST ? dstOffset : 0)); 
}

void setup() {
  Serial.begin(115200); 
//  Serial.setDebugOutput(true); 
  Serial.println("\n...\n"); 
  
  WiFi.disconnect();  // always close WiFi (after a firmware flash: a previous open connection always remains in memory)
  WiFi.begin(mySSID, myPASSWORD);
  while (WiFi.status() != WL_CONNECTED) delay(500);

  setSyncProvider(syncProvider);  // set the external time provider
  setSyncInterval(SyncInterval);  // set the number of seconds between re-sync
}

void loop() {
  char msTs[14];  // timestamp String with millis
  strcpy(msTs, tms.getMsTimestamp());  // usable as websocket timestamp or ...
  time_t u = now();
  time_t t = localTime(u);
  printf("Loc time   : %02u-%02u-%04u %02u:%02u:%02u \n", day(t), month(t), year(t), hour(t), minute(t), second(t));
  printf("UTC web    : %02u-%02u-%04u %02u:%02u:%02u \n", day(u), month(u), year(u), hour(u), minute(u), second(u));
  printf("UTC Millis$: %s \n\n", msTs);
  delay(10000);
}
  
